
// HL2 Basis

//Global multipliers
#define GLOBAL_SCALE 1.4
#define GLOBAL_TEXTURE 1.4

#define NORMFACTOR 0.690988

void HL2NormVP(	float4 position	: POSITION,
				float2 normalUV	: TEXCOORD0,
				float2 lightUV	: TEXCOORD1,
				
				
							
				out float4 oPosition : POSITION,
				out float2 onormalUV : TEXCOORD0,
				out float2 olightUV : TEXCOORD1,
				
				uniform float4x4 worldViewProj

            )

{	
	oPosition = mul(worldViewProj, position);
	
	onormalUV = normalUV;
	olightUV = lightUV;

}
//------------------------------------------------------------------------------

void HL2NormFP(	float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D albedo : register(s0),
				uniform sampler2D normal : register(s1),
				uniform sampler2D coeff0 : register(s2),
				uniform sampler2D coeff1 : register(s3),
				uniform sampler2D coeff2 : register(s4)
)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //srgb to linear, sRGB state does not work in Ogre so we do our own conversion
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 basis1 = float3(0.8165, 0.0, 0.577);
		float3 basis2 = float3(-0.408, 0.707, 0.577);
		float3 basis3 = float3(-0.408, -0.707, 0.577);
		
		
		float3 Irr = NORMFACTOR * (tex2D(coeff0,lightUV) * dot(basis1, norm) 
			     + tex2D(coeff1,lightUV) * dot(basis2, norm)	
			     + tex2D(coeff2,lightUV) * dot(basis3, norm));
											
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;
		color.a = 1.0;				
}


void HL2NormNoAlbedoFP(	float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D normal : register(s0),
				uniform sampler2D coeff0 : register(s1),
				uniform sampler2D coeff1 : register(s2),
				uniform sampler2D coeff2 : register(s3)
)
{
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 basis1 = float3(0.8165, 0.0, 0.577);
		float3 basis2 = float3(-0.408, 0.707, 0.577);
		float3 basis3 = float3(-0.408, -0.707, 0.577);
		
		
		float3 Irr = NORMFACTOR * (tex2D(coeff0,lightUV) * dot(basis1, norm) 
			     + tex2D(coeff1,lightUV) * dot(basis2, norm)	
			     + tex2D(coeff2,lightUV) * dot(basis3, norm));
										
		color.rgb = GLOBAL_SCALE*Irr;
		color.a = 1.0;				
}

