
// Spherical harmonics

//Global compress range is set by hand or defined by the maximum value in the textures [texturename]_1..8
//See baking script for compression.
#define COMPRESSRANGE 1 //compress range of -1..1

//Global multipliers
#define GLOBAL_SCALE 2.5 //2.5066282 is sqrt2pi
#define GLOBAL_TEXTURE 2.5

#define OVERSQ2PI 0.39894228
#define OVER6SQPI 1.38197659
#define OVER3SQ2PI 0.69098830
#define OVER30SQPI 3.09019361
#define OVER5SQ2PI 0.89206205

void HSHNormVP(	float4 position	: POSITION,
				float2 normalUV	: TEXCOORD0,
				float2 lightUV	: TEXCOORD1,
				
				
							
				out float4 oPosition : POSITION,
				out float2 onormalUV : TEXCOORD0,
				out float2 olightUV : TEXCOORD1,
				
				uniform float4x4 worldViewProj

            )

{	
	oPosition = mul(worldViewProj, position);
	
	onormalUV = normalUV;
	olightUV = lightUV;

}
//------------------------------------------------------------------------------

void HSHNormFP(	float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D albedo : register(s0),
				uniform sampler2D normal : register(s1),
				uniform sampler2D coeff0 : register(s2),
				uniform sampler2D coeff1 : register(s3),
				uniform sampler2D coeff2 : register(s4),
				uniform sampler2D coeff3 : register(s5),
				uniform sampler2D coeff4 : register(s6),
				uniform sampler2D coeff5 : register(s7),
				uniform sampler2D coeff6 : register(s8),
				uniform sampler2D coeff7 : register(s9),
				uniform sampler2D coeff8 : register(s10)

)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //srgb to linear, sRGB state does not work in Ogre so we do our own conversion
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQ2PI //this SHould be done with sRGB lookup
					-(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVER6SQPI*norm.y*sqrt(norm.z/(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVER3SQ2PI*(2*norm.z-1)
					-(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVER6SQPI*norm.x*sqrt(norm.z/(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff4,lightUV)-COMPRESSRANGE)*OVER30SQPI*(2*norm.x*norm.y*norm.z)/(1+norm.z)
					-(2*COMPRESSRANGE*tex2D(coeff5,lightUV)-COMPRESSRANGE)*OVER30SQPI*norm.y*(2*norm.z-1)*sqrt(norm.z/(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff6,lightUV)-COMPRESSRANGE)*OVER5SQ2PI*(6*norm.z*norm.z-6*norm.z+1)
					-(2*COMPRESSRANGE*tex2D(coeff7,lightUV)-COMPRESSRANGE)*OVER30SQPI*norm.x*(2*norm.z-1)*sqrt(norm.z/(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff8,lightUV)-COMPRESSRANGE)*OVER30SQPI*((norm.x*norm.x-norm.y*norm.y)*norm.z)/(1+norm.z);
											
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;
		color.a = 1.0;				
}


void HSHNormNoAlbedoFP(	float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D normal : register(s0),
				uniform sampler2D coeff0 : register(s1),
				uniform sampler2D coeff1 : register(s2),
				uniform sampler2D coeff2 : register(s3),
				uniform sampler2D coeff3 : register(s4),
				uniform sampler2D coeff4 : register(s5),
				uniform sampler2D coeff5 : register(s6),
				uniform sampler2D coeff6 : register(s7),
				uniform sampler2D coeff7 : register(s8),
				uniform sampler2D coeff8 : register(s9)
)
{
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQ2PI //this SHould be done with sRGB lookup
					-(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVER6SQPI*norm.y*sqrt(norm.z/(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVER3SQ2PI*(2*norm.z-1)
					-(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVER6SQPI*norm.x*sqrt(norm.z/(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff4,lightUV)-COMPRESSRANGE)*OVER30SQPI*(2*norm.x*norm.y*norm.z)/(1+norm.z)
					-(2*COMPRESSRANGE*tex2D(coeff5,lightUV)-COMPRESSRANGE)*OVER30SQPI*norm.y*(2*norm.z-1)*sqrt(norm.z/(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff6,lightUV)-COMPRESSRANGE)*OVER5SQ2PI*(6*norm.z*norm.z-6*norm.z+1)
					-(2*COMPRESSRANGE*tex2D(coeff7,lightUV)-COMPRESSRANGE)*OVER30SQPI*norm.x*(2*norm.z-1)*sqrt(norm.z/(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff8,lightUV)-COMPRESSRANGE)*OVER30SQPI*((norm.x*norm.x-norm.y*norm.y)*norm.z)/(1+norm.z);
										
		color.rgb = GLOBAL_SCALE*Irr;
		color.a = 1.0;				
}

void HSHNormLinFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D albedo : register(s0),
				uniform sampler2D normal : register(s1),
				uniform sampler2D coeff0 : register(s2),
				uniform sampler2D coeff1 : register(s3),
				uniform sampler2D coeff2 : register(s4),
				uniform sampler2D coeff3 : register(s5)
)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //we do our own sRGB to linear, sRGB Gamma correction in the renderwindow must be set.
		float3 norm = 2*tex2D(normal,normalUV)-1;

		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQ2PI //this SHould be done with sRGB lookup
					-(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVER6SQPI*norm.y*sqrt(norm.z/(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVER3SQ2PI*(2*norm.z-1)
					-(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVER6SQPI*norm.x*sqrt(norm.z/(1+norm.z));
										
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;
		color.a = 1.0;			
}

void HSHNormLinNoAlbedoFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D normal : register(s0),
				uniform sampler2D coeff0 : register(s1),
				uniform sampler2D coeff1 : register(s2),
				uniform sampler2D coeff2 : register(s3),
				uniform sampler2D coeff3 : register(s4)
)
{
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQ2PI //this SHould be done with sRGB lookup
					-(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVER6SQPI*norm.y*sqrt(((1-norm.z)*norm.z)/(1-norm.z*norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVER3SQ2PI*(2*norm.z-1)
					-(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVER6SQPI*norm.x*sqrt(((1-norm.z)*norm.z)/(1-norm.z*norm.z));
										
		color.rgb = GLOBAL_SCALE*Irr;
		color.a = 1.0;		
}

void HSHNormConstFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D albedo : register(s0),
				uniform sampler2D coeff0 : register(s1)
)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //we do our own sRGB to linear, sRGB Gamma correction in the renderwindow must be set.

		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQ2PI;
					
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;
		color.a = 1.0;			
}

void HSHNormConstNoAlbedoFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
				
				uniform sampler2D coeff0 : register(s0)
)
{
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQ2PI;
									
		color.rgb =GLOBAL_SCALE*Irr;
		color.a = 1.0;		
}

