
// Makhotkin basis

//Global compress range is set by hand or defined by the maximum value in the textures [texturename]_1..8
//See baking script for compression.
#define COMPRESSRANGE 1 //compress range of -1..1

//Global multipliers
#define GLOBAL_SCALE 2.5 //2.5066282 is sqrt2pi
#define GLOBAL_TEXTURE 2.5

#define OVERSQPI 0.56418958
#define OVER3SQPIx2 1.954410047
#define OVER2SQPI 0.79788456
#define OVER15SQPIx2 4.37019372
#define OVER3SQPI 0.97720502

void MakNormVP(	float4 position	: POSITION,
				float2 normalUV	: TEXCOORD0,
				float2 lightUV	: TEXCOORD1,
				
				
							
				out float4 oPosition : POSITION,
				out float2 onormalUV : TEXCOORD0,
				out float2 olightUV : TEXCOORD1,
				
				uniform float4x4 worldViewProj

            )

{	
	oPosition = mul(worldViewProj, position);
	
	onormalUV = normalUV;
	olightUV = lightUV;

}
//------------------------------------------------------------------------------

void MakNormFP(	float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D albedo : register(s0),
				uniform sampler2D normal : register(s1),
				uniform sampler2D coeff0 : register(s2),
				uniform sampler2D coeff1 : register(s3),
				uniform sampler2D coeff2 : register(s4),
				uniform sampler2D coeff3 : register(s5),
				uniform sampler2D coeff4 : register(s6),
				uniform sampler2D coeff5 : register(s7),
				uniform sampler2D coeff6 : register(s8),
				uniform sampler2D coeff7 : register(s9),
				uniform sampler2D coeff8 : register(s10)

)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //srgb to linear, sRGB state does not work in Ogre so we do our own conversion
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQPI*sqrt(norm.z) //this should be done with sRGB lookup
					+(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*(norm.x*norm.z)/sqrt(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVER2SQPI*(3*norm.z-2)*sqrt(norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*(norm.y*norm.z)/sqrt(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff4,lightUV)-COMPRESSRANGE)*OVER15SQPIx2*sqrt(norm.z*norm.z*norm.z)*(norm.x*norm.x-norm.y*norm.y)/(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff5,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*norm.x*norm.z*(5*norm.z-3)/sqrt(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff6,lightUV)-COMPRESSRANGE)*OVER3SQPI*sqrt(norm.z)*((5*norm.z-6)*2*norm.z+3)
					+(2*COMPRESSRANGE*tex2D(coeff7,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*norm.y*norm.z*(5*norm.z-3)/sqrt(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff8,lightUV)-COMPRESSRANGE)*OVER15SQPIx2*sqrt(norm.z*norm.z*norm.z)*2*(norm.x*norm.y)/(1+norm.z);
											
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;
		color.a = 1.0;	
}


void MakNormNoAlbedoFP(	float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D normal : register(s0),
				uniform sampler2D coeff0 : register(s1),
				uniform sampler2D coeff1 : register(s2),
				uniform sampler2D coeff2 : register(s3),
				uniform sampler2D coeff3 : register(s4),
				uniform sampler2D coeff4 : register(s5),
				uniform sampler2D coeff5 : register(s6),
				uniform sampler2D coeff6 : register(s7),
				uniform sampler2D coeff7 : register(s8),
				uniform sampler2D coeff8 : register(s9)
)
{
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQPI*sqrt(norm.z) //this should be done with sRGB lookup
					+(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*(norm.x*norm.z)/sqrt(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVER2SQPI*(3*norm.z-2)*sqrt(norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*(norm.y*norm.z)/sqrt(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff4,lightUV)-COMPRESSRANGE)*OVER15SQPIx2*sqrt(norm.z*norm.z*norm.z)*(norm.x*norm.x-norm.y*norm.y)/(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff5,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*norm.x*norm.z*(5*norm.z-3)/sqrt(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff6,lightUV)-COMPRESSRANGE)*OVER3SQPI*sqrt(norm.z)*((5*norm.z-6)*2*norm.z+3)
					+(2*COMPRESSRANGE*tex2D(coeff7,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*norm.y*norm.z*(5*norm.z-3)/sqrt(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff8,lightUV)-COMPRESSRANGE)*OVER15SQPIx2*sqrt(norm.z*norm.z*norm.z)*2*(norm.x*norm.y)/(1+norm.z);
										
		color.rgb = GLOBAL_SCALE*Irr;
		color.a = 1.0;				
}

void MakNormLinFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D albedo : register(s0),
				uniform sampler2D normal : register(s1),
				uniform sampler2D coeff0 : register(s2),
				uniform sampler2D coeff1 : register(s3),
				uniform sampler2D coeff2 : register(s4),
				uniform sampler2D coeff3 : register(s5)
)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //we do our own sRGB to linear, sRGB Gamma correction in the renderwindow must be set.
		float3 norm = 2*tex2D(normal,normalUV)-1;

		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQPI*sqrt(norm.z) //this should be done with sRGB lookup
					+(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*(norm.x*norm.z)/sqrt(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVER2SQPI*(3*norm.z-2)*sqrt(norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*(norm.y*norm.z)/sqrt(1+norm.z);
										
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;	
		color.a = 1.0;			
}

void MakNormLinNoAlbedoFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D normal : register(s0),
				uniform sampler2D coeff0 : register(s1),
				uniform sampler2D coeff1 : register(s2),
				uniform sampler2D coeff2 : register(s3),
				uniform sampler2D coeff3 : register(s4)
)
{
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQPI*sqrt(norm.z) //this should be done with sRGB lookup
					+(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*(norm.x*norm.z*sqrt(1+norm.z))/(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVER2SQPI*(3*norm.z-2)*sqrt(norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*(norm.y*norm.z*sqrt(1+norm.z))/(1+norm.z);

										
		color.rgb = GLOBAL_SCALE*Irr;
		color.a = 1.0;		
}

void MakNormConstFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
				
				uniform sampler2D albedo : register(s0),
				uniform sampler2D normal : register(s1),
				uniform sampler2D coeff0 : register(s2)
)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //we do our own sRGB to linear, sRGB Gamma correction in the renderwindow must be set.
	
		float3 norm = 2*tex2D(normal,normalUV)-1;
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQPI*sqrt(norm.z);
					
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;
		color.a = 1.0;			
}

void MakNormConstNoAlbedoFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
				
				uniform sampler2D normal : register(s0),
				uniform sampler2D coeff0 : register(s1)
)
{
		float3 norm = 2*tex2D(normal,normalUV)-1;
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQPI*sqrt(norm.z);
									
		color.rgb =GLOBAL_SCALE*Irr;
		color.a = 1.0;		
}

