
// Spherical harmonics

//Global compress range is set by hand or defined by the maximum value in the textures [texturename]_1..8
//See baking script for compression.
#define COMPRESSRANGE 1 //compress range of -1..1

//Global multipliers
#define GLOBAL_SCALE 2.5 //2.5066282 is sqrt2pi
#define GLOBAL_TEXTURE 2.5

#define OVERSQPIx05 0.28209479
#define OVERSQPI3x05 0.48860251
#define OVERSQPI15x05 1.09254843
#define OVERSQPI5x025 0.31539156
#define OVERSQPI15x025 0.54627421

void SHNormVP(	float4 position	: POSITION,
				float2 normalUV	: TEXCOORD0,
				float2 lightUV	: TEXCOORD1,
				
										
				out float4 oPosition : POSITION,
				out float2 onormalUV : TEXCOORD0,
				out float2 olightUV : TEXCOORD1,
				
				uniform float4x4 worldViewProj

            )

{	
	oPosition = mul(worldViewProj, position);
	
	onormalUV = normalUV;
	olightUV = lightUV;

}
//------------------------------------------------------------------------------
void SHNormVolumeVP(	float4 position	: POSITION,
						float3 normal : NORMAL,
						float3 tangent : TANGENT,
						float3 binormal : BINORMAL,
						float2 normalUV	: TEXCOORD0,
					float2 lightUV	: TEXCOORD1,
													
				out float4 oPosition : POSITION,
				out float3 onormal : NORMAL,
				out float3 otangent: TANGENT,
				out float3 obinormal: BINORMAL,
				out float2 onormalUV : TEXCOORD0,
				out float2 olightUV : TEXCOORD1,
				out float3 position3d : TEXCOORD2,
				
				uniform float4x4 worldViewProj,
				uniform float4x4 toWorld
            )

{	
	oPosition = mul(worldViewProj, position);
	onormal = normal;
	obinormal = binormal;
	otangent = tangent;
	onormalUV = normalUV;
	olightUV = lightUV;
	
	position3d = mul(toWorld,position);

}
//------------------------------------------------------------------------------

void SHNormFP(	float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D albedo : register(s0),
				uniform sampler2D normal : register(s1),
				uniform sampler2D coeff0 : register(s2),
				uniform sampler2D coeff1 : register(s3),
				uniform sampler2D coeff2 : register(s4),
				uniform sampler2D coeff3 : register(s5),
				uniform sampler2D coeff4 : register(s6),
				uniform sampler2D coeff5 : register(s7),
				uniform sampler2D coeff6 : register(s8),
				uniform sampler2D coeff7 : register(s9),
				uniform sampler2D coeff8 : register(s10)

)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //srgb to linear, sRGB state does not work in Ogre so we do our own conversion
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQPIx05 //this should be done with sRGB lookup
					+(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVERSQPI3x05*norm.y
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVERSQPI3x05*norm.z
					+(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVERSQPI3x05*norm.x
					+(2*COMPRESSRANGE*tex2D(coeff4,lightUV)-COMPRESSRANGE)*OVERSQPI15x05*norm.x*norm.y
					+(2*COMPRESSRANGE*tex2D(coeff5,lightUV)-COMPRESSRANGE)*OVERSQPI15x05*norm.z*norm.y
					+(2*COMPRESSRANGE*tex2D(coeff6,lightUV)-COMPRESSRANGE)*OVERSQPI5x025*(3*norm.z*norm.z-1)
					+(2*COMPRESSRANGE*tex2D(coeff7,lightUV)-COMPRESSRANGE)*OVERSQPI15x05*norm.z*norm.x
					+(2*COMPRESSRANGE*tex2D(coeff8,lightUV)-COMPRESSRANGE)*OVERSQPI15x025*(norm.x*norm.x - norm.y*norm.y);
											
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;
		color.a = 1.0;	
}


void SHNormNoAlbedoFP(	float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D normal : register(s0),
				uniform sampler2D coeff0 : register(s1),
				uniform sampler2D coeff1 : register(s2),
				uniform sampler2D coeff2 : register(s3),
				uniform sampler2D coeff3 : register(s4),
				uniform sampler2D coeff4 : register(s5),
				uniform sampler2D coeff5 : register(s6),
				uniform sampler2D coeff6 : register(s7),
				uniform sampler2D coeff7 : register(s8),
				uniform sampler2D coeff8 : register(s9)
)
{
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQPIx05 //this should be done with sRGB lookup
					+(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVERSQPI3x05*norm.y
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVERSQPI3x05*norm.z
					+(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVERSQPI3x05*norm.x
					+(2*COMPRESSRANGE*tex2D(coeff4,lightUV)-COMPRESSRANGE)*OVERSQPI15x05*norm.x*norm.y
					+(2*COMPRESSRANGE*tex2D(coeff5,lightUV)-COMPRESSRANGE)*OVERSQPI15x05*norm.z*norm.y
					+(2*COMPRESSRANGE*tex2D(coeff6,lightUV)-COMPRESSRANGE)*OVERSQPI5x025*(3*norm.z*norm.z-1)
					+(2*COMPRESSRANGE*tex2D(coeff7,lightUV)-COMPRESSRANGE)*OVERSQPI15x05*norm.z*norm.x
					+(2*COMPRESSRANGE*tex2D(coeff8,lightUV)-COMPRESSRANGE)*OVERSQPI15x025*(norm.x*norm.x - norm.y*norm.y);
										
		color.rgb = GLOBAL_SCALE*Irr;
		color.a = 1.0;				
}

void SHNormLinFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D albedo : register(s0),
				uniform sampler2D normal : register(s1),
				uniform sampler2D coeff0 : register(s2),
				uniform sampler2D coeff1 : register(s3),
				uniform sampler2D coeff2 : register(s4),
				uniform sampler2D coeff3 : register(s5)
)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //we do our own sRGB to linear, sRGB Gamma correction in the renderwindow must be set.
		float3 norm = 2*tex2D(normal,normalUV)-1;

		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQPIx05 //this should be done with sRGB lookup
					+(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVERSQPI3x05*norm.y
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVERSQPI3x05*norm.z
					+(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVERSQPI3x05*norm.x;
										
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;	
		color.a = 1.0;			
}

void SHNormLinNoAlbedoFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D normal : register(s0),
				uniform sampler2D coeff0 : register(s1),
				uniform sampler2D coeff1 : register(s2),
				uniform sampler2D coeff2 : register(s3),
				uniform sampler2D coeff3 : register(s4)
)
{
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQPIx05 //this should be done with sRGB lookup
					+(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVERSQPI3x05*norm.y
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVERSQPI3x05*norm.z
					+(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVERSQPI3x05*norm.x;

										
		color.rgb = GLOBAL_SCALE*Irr;
		color.a = 1.0;		
}

void SHNormConstFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D albedo : register(s0),
				uniform sampler2D coeff0 : register(s1)
)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //we do our own sRGB to linear, sRGB Gamma correction in the renderwindow must be set.

		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQPIx05;
					
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;
		color.a = 1.0;			
}

void SHNormConstNoAlbedoFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
				
				uniform sampler2D coeff0 : register(s0)
)
{
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQPIx05;
									
		color.rgb =GLOBAL_SCALE*Irr;
		color.a = 1.0;		
}
//---------------------------------------------------------------------------
void SHNormVolumeFP(float3 normal: NORMAL,
				float3 tangent: TANGENT,
				float3 binormal: BINORMAL,
				float2 normalUV : TEXCOORD0,
				float2 lightUV : TEXCOORD1,
				float3 position3d : TEXCOORD2,
				
				out float4 color	: COLOR,
				
				uniform sampler2D albedo : register(s0),
				uniform sampler2D normal0 : register(s1),
				uniform sampler2D coeff0 : register(s2),
				uniform sampler3D volume0 : register(s3),
				uniform sampler3D volume1 : register(s4),
				uniform sampler3D volume2 : register(s5),
				uniform sampler3D volume3 : register(s6),
				uniform sampler3D volume4 : register(s7),
				uniform sampler3D volume5 : register(s8),
				uniform sampler3D volume6 : register(s9),
				uniform sampler3D volume7 : register(s10),
				uniform sampler3D volume8 : register(s11),
				uniform float4x4 toWorld
)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2);
		
		float3 ts_normal = 2*tex2D(normal0,normalUV)-1;

		normal = mul(normal, toWorld);
		normal = normalize(normal);
		float3 ws_normal = tangent * ts_normal.x + binormal * ts_normal.y + normal * ts_normal.z;
		float3 width = float3(52.5f,13.125f,52.5f);
		float3 offset = float3(2.0f,9.0f,0.5f);
		float3 lightUVW = position3d + (width/2);
		lightUVW += offset;
		lightUVW.x/=width.x;
		lightUVW.y/=width.y;
		lightUVW.z/=width.z;

		lightUVW.z = -lightUVW.z;
		
		lightUVW = lightUVW.zxy;

		ws_normal.xyz = ws_normal.xzy;
		ws_normal.y = -ws_normal.y; 
		float3 Irr = pow(tex3D(volume0,lightUVW),2.2)*OVERSQPIx05 //this should be done with sRGB lookup
					+(2*COMPRESSRANGE*tex3D(volume1,lightUVW)-COMPRESSRANGE)*OVERSQPI3x05*(ws_normal.y)
					+(2*COMPRESSRANGE*tex3D(volume2,lightUVW)-COMPRESSRANGE)*OVERSQPI3x05*(ws_normal.z)
					+(2*COMPRESSRANGE*tex3D(volume3,lightUVW)-COMPRESSRANGE)*OVERSQPI3x05*(ws_normal.x)
					+(2*COMPRESSRANGE*tex3D(volume4,lightUVW)-COMPRESSRANGE)*OVERSQPI15x05*ws_normal.x*ws_normal.y
					+(2*COMPRESSRANGE*tex3D(volume5,lightUVW)-COMPRESSRANGE)*OVERSQPI15x05*ws_normal.z*ws_normal.y
					+(2*COMPRESSRANGE*tex3D(volume6,lightUVW)-COMPRESSRANGE)*OVERSQPI5x025*(3*ws_normal.z*ws_normal.z-1)
					+(2*COMPRESSRANGE*tex3D(volume7,lightUVW)-COMPRESSRANGE)*OVERSQPI15x05*ws_normal.z*ws_normal.x
					+(2*COMPRESSRANGE*tex3D(volume8,lightUVW)-COMPRESSRANGE)*OVERSQPI15x025*(ws_normal.x*ws_normal.x - ws_normal.y*ws_normal.y);						

		color.rgb =GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;
		color.a = 1.0;		
}
//---------------------------------------------------------------------------
void SHNormVolumeNoAlbedoFP(float3 normal: NORMAL,
				float3 tangent: TANGENT,
				float3 binormal: BINORMAL,
				float2 normalUV : TEXCOORD0,
				float2 lightUV : TEXCOORD1,
				float3 position3d : TEXCOORD2,
				
				out float4 color	: COLOR,
				
				uniform sampler2D normal0 : register(s0),
				uniform sampler2D coeff0 : register(s1),
				uniform sampler3D volume0 : register(s2),
				uniform sampler3D volume1 : register(s3),
				uniform sampler3D volume2 : register(s4),
				uniform sampler3D volume3 : register(s5),
				uniform sampler3D volume4 : register(s6),
				uniform sampler3D volume5 : register(s7),
				uniform sampler3D volume6 : register(s8),
				uniform sampler3D volume7 : register(s9),
				uniform sampler3D volume8 : register(s10),
				uniform float4x4 toWorld
)
{
		float3 ts_normal = 2*tex2D(normal0,normalUV)-1;

		normal = mul(normal, toWorld);
		normal = normalize(normal);
		float3 ws_normal = tangent * ts_normal.x + binormal * ts_normal.y + normal * ts_normal.z;
		float3 width = float3(52.5f,13.125f,52.5f);
		float3 offset = float3(2.0f,9.0f,0.5f);
		float3 lightUVW = position3d + (width/2);
		lightUVW += offset;
		lightUVW.x/=width.x;
		lightUVW.y/=width.y;
		lightUVW.z/=width.z;

		lightUVW.z = -lightUVW.z;
		
		lightUVW = lightUVW.zxy;

		ws_normal.xyz = ws_normal.xzy;
		ws_normal.y = -ws_normal.y; 
		
		float3 Irr = pow(tex3D(volume0,lightUVW),2.2)*OVERSQPIx05 //this should be done with sRGB lookup
					+(2*COMPRESSRANGE*tex3D(volume1,lightUVW)-COMPRESSRANGE)*OVERSQPI3x05*(ws_normal.y)
					+(2*COMPRESSRANGE*tex3D(volume2,lightUVW)-COMPRESSRANGE)*OVERSQPI3x05*(ws_normal.z)
					+(2*COMPRESSRANGE*tex3D(volume3,lightUVW)-COMPRESSRANGE)*OVERSQPI3x05*(ws_normal.x)
					+(2*COMPRESSRANGE*tex3D(volume4,lightUVW)-COMPRESSRANGE)*OVERSQPI15x05*ws_normal.x*ws_normal.y
					+(2*COMPRESSRANGE*tex3D(volume5,lightUVW)-COMPRESSRANGE)*OVERSQPI15x05*ws_normal.z*ws_normal.y
					+(2*COMPRESSRANGE*tex3D(volume6,lightUVW)-COMPRESSRANGE)*OVERSQPI5x025*(3*ws_normal.z*ws_normal.z-1)
					+(2*COMPRESSRANGE*tex3D(volume7,lightUVW)-COMPRESSRANGE)*OVERSQPI15x05*ws_normal.z*ws_normal.x
					+(2*COMPRESSRANGE*tex3D(volume8,lightUVW)-COMPRESSRANGE)*OVERSQPI15x025*(ws_normal.x*ws_normal.x - ws_normal.y*ws_normal.y);
		
		color.rgb = GLOBAL_SCALE*Irr;
		color.a = 1.0;		
}