// Zernike basis

//Global compress range is set by hand or defined by the maximum value in the textures [texturename]_1..5
//See baking script for compression.
#define COMPRESSRANGE 1 //compress range of -1..1

//Global multipliers
#define GLOBAL_SCALE 2.5 //2.5066282 is sqrt2pi
#define GLOBAL_TEXTURE 2.5

#define OVERSQ2PI 0.39894228
#define OVERSQPIx2 1.12837916
#define OVER3SQPIx2 1.95441004
#define	OVER3SQ2PI 0.69098829

void ZernNormVP(	float4 position	: POSITION,
				float2 normalUV	: TEXCOORD0,
				float2 lightUV	: TEXCOORD1,
				
				
							
				out float4 oPosition : POSITION,
				out float2 onormalUV : TEXCOORD0,
				out float2 olightUV : TEXCOORD1,
				
				uniform float4x4 worldViewProj

            )

{	
	oPosition = mul(worldViewProj, position);
	
	onormalUV = normalUV;
	olightUV = lightUV;

}

//------------------------------------------------------------------------------

void ZernNormFP(	float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D albedo : register(s0),
				uniform sampler2D normal : register(s1),
				uniform sampler2D coeff0 : register(s2),
				uniform sampler2D coeff1 : register(s3),
				uniform sampler2D coeff2 : register(s4),
				uniform sampler2D coeff3 : register(s5),
				uniform sampler2D coeff4 : register(s6),
				uniform sampler2D coeff5 : register(s7)

)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //srgb to linear, sRGB state does not work in Ogre so we do our own conversion
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQ2PI
					-(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVERSQPIx2*norm.y/sqrt(2*(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVERSQPIx2*norm.x/sqrt(2*(1+norm.z))
					-(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*norm.x*norm.y/(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff4,lightUV)-COMPRESSRANGE)*OVER3SQ2PI*(1-2*norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff5,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*(norm.x*norm.x-norm.y*norm.y)/(2*(1+norm.z));
					
										
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;
		color.a = 1.0;				
}



void ZernNormNoAlbedoFP(	float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D normal : register(s0),
				uniform sampler2D coeff0 : register(s1),
				uniform sampler2D coeff1 : register(s2),
				uniform sampler2D coeff2 : register(s3),
				uniform sampler2D coeff3 : register(s4),
				uniform sampler2D coeff4 : register(s5),
				uniform sampler2D coeff5 : register(s6)

)
{
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQ2PI
					-(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVERSQPIx2*norm.y/sqrt(2*(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVERSQPIx2*norm.x/sqrt(2*(1+norm.z))
					-(2*COMPRESSRANGE*tex2D(coeff3,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*norm.x*norm.y/(1+norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff4,lightUV)-COMPRESSRANGE)*OVER3SQ2PI*(1-2*norm.z)
					+(2*COMPRESSRANGE*tex2D(coeff5,lightUV)-COMPRESSRANGE)*OVER3SQPIx2*(norm.x*norm.x-norm.y*norm.y)/(2*(1+norm.z));
										
		color.rgb = GLOBAL_SCALE*Irr;
		color.a = 1.0;				
}

void ZernNormLinFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D albedo : register(s0),
				uniform sampler2D normal : register(s1),
				uniform sampler2D coeff0 : register(s2),
				uniform sampler2D coeff1 : register(s3),
				uniform sampler2D coeff2 : register(s4),
				uniform sampler2D coeff3 : register(s5)
)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //we do our own sRGB to linear, sRGB Gamma correction in the renderwindow must be set.
		float3 norm = 2*tex2D(normal,normalUV)-1;

		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQ2PI
					-(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVERSQPIx2*norm.y/sqrt(2*(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVERSQPIx2*norm.x/sqrt(2*(1+norm.z));
										
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;	
		color.a = 1.0;			
}

void ZernNormLinNoAlbedoFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D normal : register(s0),
				uniform sampler2D coeff0 : register(s1),
				uniform sampler2D coeff1 : register(s2),
				uniform sampler2D coeff2 : register(s3),
				uniform sampler2D coeff3 : register(s4)
)
{
		float3 norm = 2*tex2D(normal,normalUV)-1;
		
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQ2PI
					-(2*COMPRESSRANGE*tex2D(coeff1,lightUV)-COMPRESSRANGE)*OVERSQPIx2*norm.y/sqrt(2*(1+norm.z))
					+(2*COMPRESSRANGE*tex2D(coeff2,lightUV)-COMPRESSRANGE)*OVERSQPIx2*norm.x/sqrt(2*(1+norm.z));

										
		color.rgb = GLOBAL_SCALE*Irr;
		color.a = 1.0;		
}

void ZernNormConstFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
		 
				uniform sampler2D albedo : register(s0),
				uniform sampler2D coeff0 : register(s1)
)
{
		float4 alb = pow(tex2D(albedo,normalUV),2.2); //we do our own sRGB to linear, sRGB Gamma correction in the renderwindow must be set.

		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQ2PI;
					
		color.rgb = GLOBAL_SCALE*Irr*alb.rgb*GLOBAL_TEXTURE;
		color.a = 1.0;			
}

void ZernNormConstNoAlbedoFP(float2 normalUV      : TEXCOORD0,
				float2 lightUV       : TEXCOORD1,
				
				out float4 color	: COLOR,
				
				uniform sampler2D coeff0 : register(s0)
)
{
		float3 Irr = pow(tex2D(coeff0,lightUV),2.2)*OVERSQ2PI;
									
		color.rgb = GLOBAL_SCALE*Irr;
		color.a = 1.0;		
}

