#include "ExampleApplication.h"
#include "PrecomputedLightingFrameListener.h"
#include "OgreMaxScene.hpp"




using namespace OgreMax;

class SPLApplication : public ExampleApplication
{
protected:
public:
    SPLApplication()
    {
    }

    ~SPLApplication() 
    {

		
    }
protected:

    SPLFrameListener* mSPLFrameListener;

	Ogre::RenderWindow* getRenderWindow(void) const { return mWindow; }
	ConfigFile ConfigFile;

    Ogre::AnimationState * myAnim; // Animation

	void createFrameListener(void)
    {
		 // Create the FrameListener

        mSPLFrameListener = new SPLFrameListener(mWindow, mCamera, mSceneMgr, myAnim);
		mFrameListener = mSPLFrameListener; //pass down Pointer
        mRoot->addFrameListener(mFrameListener);
		mSPLFrameListener->showDebugOverlay(true);
		mSPLFrameListener->showHelpOverlay(true);
    }

	virtual void createCamera(void)
    {
        // Create the camera
        mCamera = mSceneMgr->createCamera("PlayerCam");
        // Position it at 500 in Z direction
        mCamera->setPosition(Vector3(0,5,20));
        // Look back along -Z
        mCamera->lookAt(Vector3(0,0,0));
        mCamera->setNearClipDistance(2);
		
    }

    void createScene(void)
    {
		ConfigFile.load("settings.cfg");
		String sceneName = ConfigFile.getSetting("SceneName", "General");
		String skyBoxMaterial = ConfigFile.getSetting("SkyBoxMaterial", "General");
		String animated = ConfigFile.getSetting("IsAnimated", "General");
		String animationName = ConfigFile.getSetting("AnimationName", "General");
		int isAnimated = animated == "True" ? 1 : 0;
		//Scene load
		OgreMaxScene* Scene1 = new OgreMaxScene;
		Scene1->Load(sceneName,mWindow,0,mSceneMgr,mSceneMgr->getRootSceneNode());
		if (isAnimated) 
		{
			myAnim = mSceneMgr->getAnimationState(animationName);
			myAnim->setLoop(true);
			myAnim->setEnabled(true);	
		} else 
		{
			myAnim = NULL;
		}
		//default states=best looking states
		MaterialManager::getSingleton().setDefaultTextureFiltering(TFO_ANISOTROPIC);
		MaterialManager::getSingleton().setDefaultAnisotropy(8);

		//Material Generation
		// H-basis materials
		MaterialPtr HNormMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HNorm");
		if (HNormMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HNorm material not found","createScene");
        }

		MaterialPtr HNormNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HNormNoAlbedo");
		if (HNormNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HNormNoAlbedo material not found","createScene");
        }

		MaterialPtr HNormLinMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HNormLin");
		if (HNormLinMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HNormLin material not found","createScene");
        }

		MaterialPtr HNormLinNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HNormLinNoAlbedo");
		if (HNormLinNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HNormLinNoAlbedo material not found","createScene");
        }

		MaterialPtr HNormConstMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HNormConst");
		if (HNormConstMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HNormConst material not found","createScene");
        }

		MaterialPtr HNormConstNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HNormConstNoAlbedo");
		if (HNormConstNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HNormConstNoAlbedo material not found","createScene");
        }

		// Zernike basis materials
		MaterialPtr ZernNormMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("ZernNorm");
		if (ZernNormMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"ZernNorm material not found","createScene");
        }

		MaterialPtr ZernNormNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("ZernNormNoAlbedo");
		if (ZernNormNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"ZernNormNoAlbedo material not found","createScene");
        }

		MaterialPtr ZernNormLinMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("ZernNormLin");
		if (ZernNormLinMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"ZernNormLin material not found","createScene");
        }

		MaterialPtr ZernNormLinNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("ZernNormLinNoAlbedo");
		if (ZernNormLinNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"ZernNormLinNoAlbedo material not found","createScene");
        }

		MaterialPtr ZernNormConstMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("ZernNormConst");
		if (ZernNormConstMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HNormConst material not found","createScene");
        }

		MaterialPtr ZernNormConstNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("ZernNormConstNoAlbedo");
		if (ZernNormConstNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"ZernNormConstNoAlbedo material not found","createScene");
        }
		

		// Irradiance Parallax Occlusion Mapping materials
		MaterialPtr IPOMMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("IPOM");
		if (IPOMMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"IPOM material not found","createScene");
        }

		MaterialPtr IPOMNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("IPOMNoAlbedo");
		if (IPOMNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"IPOMNoAlbedo material not found","createScene");
        }

		// Modified H-basis materials
		MaterialPtr HmodNormMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HmodNorm");
		if (HmodNormMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HmodNorm material not found","createScene");
        }

		MaterialPtr HmodNormNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HmodNormNoAlbedo");
		if (HmodNormNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HmodNormNoAlbedo material not found","createScene");
        }

		MaterialPtr HmodNormLinMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HmodNormLin");
		if (HmodNormLinMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HmodNormLin material not found","createScene");
        }

		MaterialPtr HmodNormLinNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HmodNormLinNoAlbedo");
		if (HmodNormLinNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HmodNormLinNoAlbedo material not found","createScene");
        }

		MaterialPtr HmodNormConstMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HmodNormConst");
		if (HmodNormConstMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HmodNormConst material not found","createScene");
        }

		MaterialPtr HmodNormConstNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HmodNormConstNoAlbedo");
		if (HmodNormConstNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HmodNormConstNoAlbedo material not found","createScene");
        }
		// Irradiance Volumes / Spherical Harmonics
		MaterialPtr SHNormVolumeMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("SHNormVolume");
		if (SHNormVolumeMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"SHNormVolume material not found","createScene");
        }
		MaterialPtr SHNormVolumeNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("SHNormVolumeNoAlbedo");
		if (SHNormVolumeNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"SHNormVolumeNoAlbedo material not found","createScene");
        }

		// Spherical Harmonics materials
		MaterialPtr SHNormMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("SHNorm");
		if (SHNormMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"SHNorm material not found","createScene");
        }

		MaterialPtr SHNormNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("SHNormNoAlbedo");
		if (SHNormNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"SHNormNoAlbedo material not found","createScene");
        }

		MaterialPtr SHNormLinMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("SHNormLin");
		if (SHNormLinMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"SHNormLin material not found","createScene");
        }

		MaterialPtr SHNormLinNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("SHNormLinNoAlbedo");
		if (SHNormLinNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"SHNormLinNoAlbedo material not found","createScene");
        }

		MaterialPtr SHNormConstMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("SHNormConst");
		if (SHNormConstMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"SHNormConst material not found","createScene");
        }

		MaterialPtr SHNormConstNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("SHNormConstNoAlbedo");
		if (SHNormConstNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"SHNormConstNoAlbedo material not found","createScene");
        }

		// Hemispherical Harmonics materials
		MaterialPtr HSHNormMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HSHNorm");
		if (HSHNormMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HSHNorm material not found","createScene");
        }

		MaterialPtr HSHNormNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HSHNormNoAlbedo");
		if (HSHNormNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HSHNormNoAlbedo material not found","createScene");
        }

		MaterialPtr HSHNormLinMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HSHNormLin");
		if (HSHNormLinMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HSHNormLin material not found","createScene");
        }

		MaterialPtr HSHNormLinNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HSHNormLinNoAlbedo");
		if (HSHNormLinNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HSHNormLinNoAlbedo material not found","createScene");
        }

		MaterialPtr HSHNormConstMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HSHNormConst");
		if (HSHNormConstMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HSHNormConst material not found","createScene");
        }

		MaterialPtr HSHNormConstNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HSHNormConstNoAlbedo");
		if (HSHNormConstNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HSHNormConstNoAlbedo material not found","createScene");
        }

		// Makhotkin basis materials
		MaterialPtr MakNormMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("MakNorm");
		if (MakNormMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"MakNorm material not found","createScene");
        }

		MaterialPtr MakNormNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("MakNormNoAlbedo");
		if (MakNormNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"MakNormNoAlbedo material not found","createScene");
        }

		MaterialPtr MakNormLinMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("MakNormLin");
		if (MakNormLinMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"MakNormLin material not found","createScene");
        }

		MaterialPtr MakNormLinNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("MakNormLinNoAlbedo");
		if (MakNormLinNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"MakNormLinNoAlbedo material not found","createScene");
        }

		MaterialPtr MakNormConstMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("MakNormConst");
		if (MakNormConstMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"MakNormConst material not found","createScene");
        }

		MaterialPtr MakNormConstNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("MakNormConstNoAlbedo");
		if (MakNormConstNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"MakNormConstNoAlbedo material not found","createScene");
        }

		// HL2 basis materials	
		MaterialPtr HL2NormMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HL2Norm");
		if (HL2NormMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HL2Norm material not found","createScene");
        }

		MaterialPtr HL2NormNoAlbedoMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName("HL2NormNoAlbedo");
		if (HL2NormNoAlbedoMaterial.isNull()) {
			 Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND,"HL2NormNoAlbedo material not found","createScene");
        }

		SceneManager::MovableObjectIterator EntityIterator = mSceneMgr->getMovableObjectIterator("Entity");
		Entity * currentEntity = NULL;

		while( EntityIterator.hasMoreElements() )
		{
			MaterialPtr currentMaterial;
			
			currentEntity = static_cast<Ogre::Entity *>(EntityIterator.peekNextValue());
			String currentEntityName=currentEntity->getName();
			
			bool animated = false;

			String mayaTexture=currentEntity->getSubEntity(0)->getMaterial()->getTechnique(0)->getPass(0)->getTextureUnitState(0)->getTextureName();
			String mayaBaseName;
			String mayaExtension;
			Ogre::StringUtil::splitBaseFilename(mayaTexture,mayaBaseName,mayaExtension);
 		
		    AliasTextureNamePairList currentAlias;

			currentAlias["AlbedoMap"] = mayaTexture;
			currentAlias["NormalMap"] = mayaBaseName+"Normal."+mayaExtension;

			String currentExtension = "png";

			// H-basis coefficient maps
			currentAlias["Coeff0"] = "Hbasis_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "Hbasis_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "Hbasis_" + currentEntityName + "_2."+currentExtension;
			currentAlias["Coeff3"] = "Hbasis_" + currentEntityName + "_3."+currentExtension;
			currentAlias["Coeff4"] = "Hbasis_" + currentEntityName + "_4."+currentExtension;
			currentAlias["Coeff5"] = "Hbasis_" + currentEntityName + "_5."+currentExtension;

			//full material
			currentMaterial = HNormMaterial->clone(currentEntityName+"HNormMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			
			//noalbedo material
			currentMaterial = HNormNoAlbedoMaterial->clone(currentEntityName+"HNormNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//linear versions
			currentMaterial = HNormLinMaterial->clone(currentEntityName+"HNormLinMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = HNormLinNoAlbedoMaterial->clone(currentEntityName+"HNormLinNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//lightmap versions
			currentMaterial = HNormConstMaterial->clone(currentEntityName+"HNormConstMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = HNormConstNoAlbedoMaterial->clone(currentEntityName+"HNormConstNoAlbedoMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentAlias.clear();
			currentAlias["AlbedoMap"] = mayaTexture;
			currentAlias["NormalMap"] = mayaBaseName+"Normal."+mayaExtension;
			currentAlias["HeightMap"] = mayaBaseName+"Height."+mayaExtension;

			// modified H-basis coefficient maps
			currentAlias["Coeff0"] = "modHbasis_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "modHbasis_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "modHbasis_" + currentEntityName + "_2."+currentExtension;
			currentAlias["Coeff3"] = "modHbasis_" + currentEntityName + "_3."+currentExtension;
			currentAlias["Coeff4"] = "modHbasis_" + currentEntityName + "_4."+currentExtension;
			currentAlias["Coeff5"] = "modHbasis_" + currentEntityName + "_5."+currentExtension;

			// Parallax
			currentMaterial = IPOMMaterial->clone(currentEntityName+"IPOMMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			currentMaterial = IPOMNoAlbedoMaterial->clone(currentEntityName+"IPOMNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//full material
			currentMaterial = HmodNormMaterial->clone(currentEntityName+"HmodNormMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//set this as the starting material
			currentEntity->setMaterial(currentMaterial);
			
			//noalbedo material
			currentMaterial = HmodNormNoAlbedoMaterial->clone(currentEntityName+"HmodNormNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//linear versions
			currentMaterial = HmodNormLinMaterial->clone(currentEntityName+"HmodNormLinMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = HmodNormLinNoAlbedoMaterial->clone(currentEntityName+"HmodNormLinNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//lightmap versions
			currentMaterial = HmodNormConstMaterial->clone(currentEntityName+"HmodNormConstMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = HmodNormConstNoAlbedoMaterial->clone(currentEntityName+"HmodNormConstNoAlbedoMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);
			
			currentAlias.clear();

			currentAlias["AlbedoMap"] = mayaTexture;
			currentAlias["NormalMap"] = mayaBaseName+"Normal."+mayaExtension;

			// Spherical harmonics coefficient maps
			currentAlias["Coeff0"] = "SH_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "SH_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "SH_" + currentEntityName + "_2."+currentExtension;
			currentAlias["Coeff3"] = "SH_" + currentEntityName + "_3."+currentExtension;
			currentAlias["Coeff4"] = "SH_" + currentEntityName + "_4."+currentExtension;
			currentAlias["Coeff5"] = "SH_" + currentEntityName + "_5."+currentExtension;
			currentAlias["Coeff6"] = "SH_" + currentEntityName + "_6."+currentExtension;
			currentAlias["Coeff7"] = "SH_" + currentEntityName + "_7."+currentExtension;
			currentAlias["Coeff8"] = "SH_" + currentEntityName + "_8."+currentExtension;
			currentAlias["Volume0"] = "volume_sh_0.dds";
			currentAlias["Volume1"] = "volume_sh_1.dds";
			currentAlias["Volume2"] = "volume_sh_2.dds";
			currentAlias["Volume3"] = "volume_sh_3.dds";
			currentAlias["Volume4"] = "volume_sh_4.dds";
			currentAlias["Volume5"] = "volume_sh_5.dds";
			currentAlias["Volume6"] = "volume_sh_6.dds";
			currentAlias["Volume7"] = "volume_sh_7.dds";
			currentAlias["Volume8"] = "volume_sh_8.dds";

			//full material
			currentMaterial = SHNormMaterial->clone(currentEntityName+"SHNormMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			
			//noalbedo material
			currentMaterial = SHNormNoAlbedoMaterial->clone(currentEntityName+"SHNormNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			
			//linear versions
			currentMaterial = SHNormLinMaterial->clone(currentEntityName+"SHNormLinMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = SHNormLinNoAlbedoMaterial->clone(currentEntityName+"SHNormLinNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//lightmap versions
			currentMaterial = SHNormConstMaterial->clone(currentEntityName+"SHNormConstMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = SHNormConstNoAlbedoMaterial->clone(currentEntityName+"SHNormConstNoAlbedoMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);
			
			// Volumetric versions
			currentMaterial = SHNormVolumeMaterial->clone(currentEntityName+"SHNormVolumeMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);
			if (animated == true)
				currentEntity->setMaterial(currentMaterial);

			currentMaterial = SHNormVolumeNoAlbedoMaterial->clone(currentEntityName+"SHNormVolumeNoAlbedoMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);


			currentAlias.clear();

			currentAlias["AlbedoMap"] = mayaTexture;
			currentAlias["NormalMap"] = mayaBaseName+"Normal."+mayaExtension;

			// Makhotkin basis coefficient maps
			currentAlias["Coeff0"] = "Mak_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "Mak_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "Mak_" + currentEntityName + "_2."+currentExtension;
			currentAlias["Coeff3"] = "Mak_" + currentEntityName + "_3."+currentExtension;
			currentAlias["Coeff4"] = "Mak_" + currentEntityName + "_4."+currentExtension;
			currentAlias["Coeff5"] = "Mak_" + currentEntityName + "_5."+currentExtension;
			currentAlias["Coeff6"] = "Mak_" + currentEntityName + "_6."+currentExtension;
			currentAlias["Coeff7"] = "Mak_" + currentEntityName + "_7."+currentExtension;
			currentAlias["Coeff8"] = "Mak_" + currentEntityName + "_8."+currentExtension;
			
			//full material
			currentMaterial = MakNormMaterial->clone(currentEntityName+"MakNormMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			
			//noalbedo material
			currentMaterial = MakNormNoAlbedoMaterial->clone(currentEntityName+"MakNormNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			
			//linear versions
			currentMaterial = MakNormLinMaterial->clone(currentEntityName+"MakNormLinMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MakNormLinNoAlbedoMaterial->clone(currentEntityName+"MakNormLinNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//lightmap versions
			currentMaterial = MakNormConstMaterial->clone(currentEntityName+"MakNormConstMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MakNormConstNoAlbedoMaterial->clone(currentEntityName+"MakNormConstNoAlbedoMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentAlias.clear();

			currentAlias["AlbedoMap"] = mayaTexture;
			currentAlias["NormalMap"] = mayaBaseName+"Normal."+mayaExtension;

			// Hemispherical harmonics coefficient maps
			currentAlias["Coeff0"] = "HSH_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "HSH_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "HSH_" + currentEntityName + "_2."+currentExtension;
			currentAlias["Coeff3"] = "HSH_" + currentEntityName + "_3."+currentExtension;
			currentAlias["Coeff4"] = "HSH_" + currentEntityName + "_4."+currentExtension;
			currentAlias["Coeff5"] = "HSH_" + currentEntityName + "_5."+currentExtension;
			currentAlias["Coeff6"] = "HSH_" + currentEntityName + "_6."+currentExtension;
			currentAlias["Coeff7"] = "HSH_" + currentEntityName + "_7."+currentExtension;
			currentAlias["Coeff8"] = "HSH_" + currentEntityName + "_8."+currentExtension;
			
			//full material
			currentMaterial = HSHNormMaterial->clone(currentEntityName+"HSHNormMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//noalbedo material
			currentMaterial = HSHNormNoAlbedoMaterial->clone(currentEntityName+"HSHNormNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			
			//linear versions
			currentMaterial = HSHNormLinMaterial->clone(currentEntityName+"HSHNormLinMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = HSHNormLinNoAlbedoMaterial->clone(currentEntityName+"HSHNormLinNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//lightmap versions
			currentMaterial = HSHNormConstMaterial->clone(currentEntityName+"HSHNormConstMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = HSHNormConstNoAlbedoMaterial->clone(currentEntityName+"HSHNormConstNoAlbedoMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentAlias.clear();

			currentAlias["AlbedoMap"] = mayaTexture;
			currentAlias["NormalMap"] = mayaBaseName+"Normal."+mayaExtension;

			// Zernike basis coefficient maps
			currentAlias["Coeff0"] = "Zern_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "Zern_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "Zern_" + currentEntityName + "_2."+currentExtension;
			currentAlias["Coeff3"] = "Zern_" + currentEntityName + "_3."+currentExtension;
			currentAlias["Coeff4"] = "Zern_" + currentEntityName + "_4."+currentExtension;
			currentAlias["Coeff5"] = "Zern_" + currentEntityName + "_5."+currentExtension;
			
			//full material
			currentMaterial = ZernNormMaterial->clone(currentEntityName+"ZernNormMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//noalbedo material
			currentMaterial = ZernNormNoAlbedoMaterial->clone(currentEntityName+"ZernNormNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			
			//linear versions
			currentMaterial = ZernNormLinMaterial->clone(currentEntityName+"ZernNormLinMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = ZernNormLinNoAlbedoMaterial->clone(currentEntityName+"ZernNormLinNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//lightmap versions
			currentMaterial = ZernNormConstMaterial->clone(currentEntityName+"ZernNormConstMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = ZernNormConstNoAlbedoMaterial->clone(currentEntityName+"ZernNormConstNoAlbedoMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentAlias.clear();

			currentAlias["AlbedoMap"] = mayaTexture;
			currentAlias["NormalMap"] = mayaBaseName+"Normal."+mayaExtension;

			// HL2 coefficient maps
			currentAlias["Coeff0"] = "HL2_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "HL2_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "HL2_" + currentEntityName + "_2."+currentExtension;
			
			//full material
			currentMaterial = HL2NormMaterial->clone(currentEntityName+"HL2NormMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//noalbedo material
			currentMaterial = HL2NormNoAlbedoMaterial->clone(currentEntityName+"HL2NormNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			EntityIterator.moveNext();
		}


		mSceneMgr->setSkyBox(true, skyBoxMaterial, 50 );
    }
};




