#include "HDRListener.h"

const String lighting = "Lighting: ";
const String albedo = "Albedo: ";
const String filtering = "Texture filtering: ";
const String compression = "Compression: ";


class SPLFrameListener : public ExampleFrameListener
{
public:
	Ogre::AnimationState* animation;
    SPLFrameListener(RenderWindow* win, Camera* cam, SceneManager *sceneMgr,Ogre::AnimationState* myAnim)
      : ExampleFrameListener(win, cam, false, false),
		mSceneMgr(sceneMgr),
		mHelpOn(true),
		mHelpOverlay(0),
		mAlbedoOn(true),
		mHDROn(false),
		mBasisType(MODHBASISQUADRATIC),
		mTextureType(PNG),
		hdrListener(0)
    {
		animation = myAnim;
		mHelpOverlay = OverlayManager::getSingleton().getByName("HelpOverlay");

		registerCompositors();
		
		lightingMethod = OverlayManager::getSingleton().getOverlayElement("LightingMethod");
		albedoState = OverlayManager::getSingleton().getOverlayElement("Albedo");
		textureFilter = OverlayManager::getSingleton().getOverlayElement("TextureFilter");
		textureCompression = OverlayManager::getSingleton().getOverlayElement("TextureCompression");
		lightingMethod->setCaption(lighting + "modified H-basis quadratic");
		albedoState->setCaption(albedo + (mAlbedoOn?"On":"Off"));
		textureFilter->setCaption(filtering + "Anisotropic");
		textureCompression->setCaption(compression + "Uncompressed");
    }
		
	virtual ~SPLFrameListener()
	{

		delete hdrListener;

		//Remove ourself as a Window listener
		WindowEventUtilities::removeWindowEventListener(mWindow, this);
		windowClosed(mWindow);
	}

	enum BasisType
	{
		MODHBASISCONSTANT, 
		MODHBASISLINEAR, 
		MODHBASISQUADRATIC,
		HBASISCONSTANT, 
		HBASISLINEAR, 
		HBASISQUADRATIC,
		SHCONSTANT, 
		SHLINEAR, 
		SHQUADRATIC,
		VOLUMESH,
		HSHQUADRATIC,
		HSHLINEAR,
		HSHCONSTANT,
		ZERNQUADRATIC,
		ZERNLINEAR,
		ZERNCONSTANT,
		MAKQUADRATIC,
		MAKLINEAR,
		MAKCONSTANT,
		HL2,
		MODHBASISIPOM
	};
	enum TextureType
	{
		PNG, 
		DXT, 
		EXR
	};




	void showHelpOverlay(bool show)
	{
		if (mHelpOverlay)
		{
			if (show)
				mHelpOverlay->show();
			else
				mHelpOverlay->hide();
		}
	}

    // Overriding the default processUnbufferedKeyInput so the key updates we define
    // later on work as intended.
    bool processUnbufferedKeyInput(const FrameEvent& evt)
	{
		if(mKeyboard->isKeyDown(OIS::KC_A))
			mTranslateVector.x = -mMoveScale;	// Move camera left

		if(mKeyboard->isKeyDown(OIS::KC_D))
			mTranslateVector.x = mMoveScale;	// Move camera RIGHT

		if(mKeyboard->isKeyDown(OIS::KC_UP) || mKeyboard->isKeyDown(OIS::KC_W) )
			mTranslateVector.z = -mMoveScale;	// Move camera forward

		if(mKeyboard->isKeyDown(OIS::KC_DOWN) || mKeyboard->isKeyDown(OIS::KC_S) )
			mTranslateVector.z = mMoveScale;	// Move camera backward

		if(mKeyboard->isKeyDown(OIS::KC_PGUP))
			mTranslateVector.y = mMoveScale;	// Move camera up

		if(mKeyboard->isKeyDown(OIS::KC_PGDOWN))
			mTranslateVector.y = -mMoveScale;	// Move camera down

		if(mKeyboard->isKeyDown(OIS::KC_RIGHT))
			mCamera->yaw(-mRotScale);

		if(mKeyboard->isKeyDown(OIS::KC_LEFT))
			mCamera->yaw(mRotScale);

		if( mKeyboard->isKeyDown(OIS::KC_ESCAPE))
			return false;
		if( mKeyboard->isKeyDown(OIS::KC_Q) && mTimeUntilNextToggle <= 0 )
		{
			mCamera->setPosition(Vector3(-40.8605,9.017,-23.60));
			// Look back along -Z
			//mCamera->lookAt(Vector3(-301,4,-50));
			mCamera->setOrientation(Quaternion(0.7071,-0.7071,0,0));
		}
       	if( mKeyboard->isKeyDown(OIS::KC_F2) && mTimeUntilNextToggle <= 0 )
		{
			mStatsOn = !mStatsOn;
			showDebugOverlay(mStatsOn);
			mTimeUntilNextToggle = 0.5;
		}

		if( mKeyboard->isKeyDown(OIS::KC_F1) && mTimeUntilNextToggle <= 0 )
		{
			mHelpOn = !mHelpOn;
			showHelpOverlay(mHelpOn);
			mTimeUntilNextToggle = 0.5;
		}

		if( mKeyboard->isKeyDown(OIS::KC_T) && mTimeUntilNextToggle <= 0 )
		{
			switch(mFiltering)
			{
			case TFO_BILINEAR:
				mFiltering = TFO_TRILINEAR;
				mAniso = 1;
				textureFilter->setCaption(filtering + "Trillinear");
				break;
			case TFO_TRILINEAR:
				mFiltering = TFO_ANISOTROPIC;
				mAniso = 8;
				textureFilter->setCaption(filtering + "Anisotropic");
				break;
			case TFO_ANISOTROPIC:
				mFiltering = TFO_BILINEAR;
				mAniso = 1;
				textureFilter->setCaption(filtering + "Bilinear");
				break;
			default: break;
			}
			MaterialManager::getSingleton().setDefaultTextureFiltering(mFiltering);
			MaterialManager::getSingleton().setDefaultAnisotropy(mAniso);

			showDebugOverlay(mStatsOn);
			mTimeUntilNextToggle = 0.5;
		}

		if(mKeyboard->isKeyDown(OIS::KC_SYSRQ) && mTimeUntilNextToggle <= 0)
		{
			std::ostringstream ss;
			ss << "screenshot_" << ++mNumScreenShots << ".png";
			mWindow->writeContentsToFile(ss.str());
			mTimeUntilNextToggle = 0.5;
			mDebugText = "Saved: " + ss.str();
		}

		if(mKeyboard->isKeyDown(OIS::KC_R) && mTimeUntilNextToggle <=0)
		{
			mSceneDetailIndex = (mSceneDetailIndex+1)%3 ;
			switch(mSceneDetailIndex) {
				case 0 : mCamera->setPolygonMode(PM_SOLID); break;
				case 1 : mCamera->setPolygonMode(PM_WIREFRAME); break;
				case 2 : mCamera->setPolygonMode(PM_POINTS); break;
			}
			mTimeUntilNextToggle = 0.5;
		}

		static bool displayCameraDetails = true;
		//if(mKeyboard->isKeyDown(OIS::KC_P) && mTimeUntilNextToggle <= 0)
		//{
		//	displayCameraDetails = !displayCameraDetails;
		//	mTimeUntilNextToggle = 0.5;
		//	if (!displayCameraDetails)
		//		mDebugText = "";
		//}
		// Print camera details
		if(displayCameraDetails)
			mDebugText = "P: " + StringConverter::toString(mCamera->getDerivedPosition()) +
						 " " + "O: " + StringConverter::toString(mCamera->getDerivedOrientation());

		if( mKeyboard->isKeyDown(OIS::KC_TAB) && mTimeUntilNextToggle <= 0 )
		{
			mAlbedoOn = !mAlbedoOn;

			changeMaterial(mBasisType,mAlbedoOn);
			
			albedoState->setCaption(albedo + (mAlbedoOn?"On":"Off"));

			mTimeUntilNextToggle = 0.5;
		}
		
		
		if( mKeyboard->isKeyDown(OIS::KC_H) && mTimeUntilNextToggle <= 0 )
		{
			mHDROn = !mHDROn;
		
			Ogre::Viewport *vp = mWindow->getViewport(0);
			Ogre::CompositorManager::getSingleton().setCompositorEnabled(vp, "HDR", mHDROn);

			mTimeUntilNextToggle = 0.5;
		}


		if( mKeyboard->isKeyDown(OIS::KC_1) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != MODHBASISQUADRATIC)
			{
				mBasisType = MODHBASISQUADRATIC;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "modified H-basis quadratic");
		}
		if( mKeyboard->isKeyDown(OIS::KC_2) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType!= MODHBASISLINEAR)
			{
				mBasisType = MODHBASISLINEAR;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "modified H-basis linear");
		}
		if( mKeyboard->isKeyDown(OIS::KC_3) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != MODHBASISCONSTANT)
			{
				mBasisType = MODHBASISCONSTANT;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "modified H-basis constant");
		}
		if( mKeyboard->isKeyDown(OIS::KC_4) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != SHQUADRATIC)
			{
				mBasisType = SHQUADRATIC;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Spherical Harmonics quadratic");
		}
		if( mKeyboard->isKeyDown(OIS::KC_5) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != SHLINEAR)
			{
				mBasisType = SHLINEAR;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Spherical Harmonics linear");
		}
		if( mKeyboard->isKeyDown(OIS::KC_6) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != SHCONSTANT)
			{
				mBasisType = SHCONSTANT;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Spherical Harmonics constant");
		}
		if( mKeyboard->isKeyDown(OIS::KC_7) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != HL2)
			{
				mBasisType = HL2;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "HL2 radiosity normal mapping");
		}
		if( mKeyboard->isKeyDown(OIS::KC_8) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != HSHQUADRATIC)
			{
				mBasisType = HSHQUADRATIC;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Hemispherical Harmonics quadratic");
		}
		if( mKeyboard->isKeyDown(OIS::KC_9) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != HSHLINEAR)
			{
				mBasisType = HSHLINEAR;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Hemispherical Harmonics linear");
		}
		if( mKeyboard->isKeyDown(OIS::KC_0) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != HSHCONSTANT)
			{
				mBasisType = HSHCONSTANT;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Hemispherical Harmonics constant");
		}
		if( mKeyboard->isKeyDown(OIS::KC_Z) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != HBASISQUADRATIC)
			{
				mBasisType = HBASISQUADRATIC;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "H-basis quadratic");
		}
		if( mKeyboard->isKeyDown(OIS::KC_X) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType!= HBASISLINEAR)
			{
				mBasisType = HBASISLINEAR;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "H-basis linear");
		}
		if( mKeyboard->isKeyDown(OIS::KC_C) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != HBASISCONSTANT)
			{
				mBasisType = HBASISCONSTANT;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "H-basis constant");
		}
		if( mKeyboard->isKeyDown(OIS::KC_I) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != ZERNQUADRATIC)
			{
				mBasisType = ZERNQUADRATIC;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Zernike basis quadratic");
		}
		if( mKeyboard->isKeyDown(OIS::KC_O) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType!= ZERNLINEAR)
			{
				mBasisType = ZERNLINEAR;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Zernike basis linear");
		}
		if( mKeyboard->isKeyDown(OIS::KC_P) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != ZERNCONSTANT)
			{
				mBasisType = ZERNCONSTANT;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Zernike basis constant");
		}
		if( mKeyboard->isKeyDown(OIS::KC_V) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != MAKQUADRATIC)
			{
				mBasisType = MAKQUADRATIC;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Makhotkin basis quadratic");
		}
		if( mKeyboard->isKeyDown(OIS::KC_B) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType!= MAKLINEAR)
			{
				mBasisType = MAKLINEAR;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Makhotkin basis linear");
		}
		if( mKeyboard->isKeyDown(OIS::KC_N) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != MAKCONSTANT)
			{
				mBasisType = MAKCONSTANT;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Makhotkin basis constant");
		}
		if( mKeyboard->isKeyDown(OIS::KC_Y) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != MODHBASISIPOM)
			{
				mBasisType = MODHBASISIPOM;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Irradiance Parallax Occlusion Mapping (mod H-basis)");
		}
		if( mKeyboard->isKeyDown(OIS::KC_L) && mTimeUntilNextToggle <= 0 )
		{
			if(mBasisType != VOLUMESH)
			{
				mBasisType = VOLUMESH;
				changeMaterial(mBasisType,mAlbedoOn);
			}
			lightingMethod->setCaption(lighting + "Irradiance volumes (SH)");
		}
		if( mKeyboard->isKeyDown(OIS::KC_U) && mTimeUntilNextToggle <= 0 )
		{
			
			switch(mTextureType)
			{
			case PNG:
				mTextureType = DXT;
				textureCompression->setCaption(compression + "DXT");
				break;
			case DXT:
				mTextureType = EXR;
				textureCompression->setCaption(compression + "EXR");
				break;
			case EXR:
				mTextureType = PNG;
				textureCompression->setCaption(compression + "Uncompressed");
			default: break;
			}
			
			setIrrMaps(mTextureType);


			mTimeUntilNextToggle = 0.5;
		
		}
		// Return true to continue rendering
		return true;
	}

    // Overriding the default processUnbufferedMouseInput so the Mouse updates we define
    // later on work as intended. 
    bool processUnbufferedMouseInput(const FrameEvent& evt)
    {
		 return ExampleFrameListener::processUnbufferedMouseInput(evt);   
    }

    bool frameStarted(const FrameEvent &evt)
    {
		if(animation != NULL)
			animation->addTime(evt.timeSinceLastFrame);
        return ExampleFrameListener::frameStarted(evt);
    }

	bool setIrrMaps(const TextureType tp)
	{
		
		SceneManager::MovableObjectIterator EntityIterator = mSceneMgr->getMovableObjectIterator("Entity");
		Entity * currentEntity = NULL;

		while( EntityIterator.hasMoreElements() )
		{

			MaterialPtr currentMaterial;
			
			currentEntity = static_cast<Ogre::Entity *>(EntityIterator.peekNextValue());
			String currentEntityName=currentEntity->getName();
	
			String currentExtension;
			if(tp == PNG) currentExtension = "png";
			else if (tp == DXT) currentExtension = "DDS";
			else if (tp == EXR) currentExtension = "exr";

			AliasTextureNamePairList currentAlias;
			
			// HL2 coefficient maps
			currentAlias["Coeff0"] = "hl2_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "hl2_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "hl2_" + currentEntityName + "_2."+currentExtension;
			
			//full material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HL2NormMat");
			currentMaterial->applyTextureAliases(currentAlias);

			//noalbedo material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HL2NormNoAlbedoMat");
			currentMaterial->applyTextureAliases(currentAlias);
			
			// H-basis coefficient maps
			currentAlias["Coeff0"] = "Hbasis_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "Hbasis_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "Hbasis_" + currentEntityName + "_2."+currentExtension;
			currentAlias["Coeff3"] = "Hbasis_" + currentEntityName + "_3."+currentExtension;
			currentAlias["Coeff4"] = "Hbasis_" + currentEntityName + "_4."+currentExtension;
			currentAlias["Coeff5"] = "Hbasis_" + currentEntityName + "_5."+currentExtension;

			//full material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HNormMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			
			//noalbedo material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HNormNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//linear versions
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HNormLinMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HNormLinNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//lightmap versions
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HNormConstMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HNormConstNoAlbedoMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentAlias.clear();

			// modified H-basis coefficient maps
			currentAlias["Coeff0"] = "modHbasis_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "modHbasis_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "modHbasis_" + currentEntityName + "_2."+currentExtension;
			currentAlias["Coeff3"] = "modHbasis_" + currentEntityName + "_3."+currentExtension;
			currentAlias["Coeff4"] = "modHbasis_" + currentEntityName + "_4."+currentExtension;
			currentAlias["Coeff5"] = "modHbasis_" + currentEntityName + "_5."+currentExtension;

			//full material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HmodNormMat");
			currentMaterial->applyTextureAliases(currentAlias);

			//noalbedo material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HmodNormNoAlbedoMat");
			currentMaterial->applyTextureAliases(currentAlias);

			//linear versions
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HmodNormLinMat");
			currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HmodNormLinNoAlbedoMat");
			currentMaterial->applyTextureAliases(currentAlias);

			//lightmap versions
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HmodNormConstMat");
		    currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HmodNormConstNoAlbedoMat");
		    currentMaterial->applyTextureAliases(currentAlias);
			
			currentAlias.clear();

			// Makhotkin basis coefficient maps
			currentAlias["Coeff0"] = "Mak_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "Mak_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "Mak_" + currentEntityName + "_2."+currentExtension;
			currentAlias["Coeff3"] = "Mak_" + currentEntityName + "_3."+currentExtension;
			currentAlias["Coeff4"] = "Mak_" + currentEntityName + "_4."+currentExtension;
			currentAlias["Coeff5"] = "Mak_" + currentEntityName + "_5."+currentExtension;
			currentAlias["Coeff6"] = "Mak_" + currentEntityName + "_6."+currentExtension;
			currentAlias["Coeff7"] = "Mak_" + currentEntityName + "_7."+currentExtension;
			currentAlias["Coeff8"] = "Mak_" + currentEntityName + "_8."+currentExtension;
			
			//full material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"MakNormMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			
			//noalbedo material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"MakNormNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			
			//linear versions
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"MakNormLinMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"MakNormLinNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//lightmap versions
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"MakNormConstMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"MakNormConstNoAlbedoMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentAlias.clear();

			// Hemispherical harmonics coefficient maps
			currentAlias["Coeff0"] = "HSH_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "HSH_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "HSH_" + currentEntityName + "_2."+currentExtension;
			currentAlias["Coeff3"] = "HSH_" + currentEntityName + "_3."+currentExtension;
			currentAlias["Coeff4"] = "HSH_" + currentEntityName + "_4."+currentExtension;
			currentAlias["Coeff5"] = "HSH_" + currentEntityName + "_5."+currentExtension;
			currentAlias["Coeff6"] = "HSH_" + currentEntityName + "_6."+currentExtension;
			currentAlias["Coeff7"] = "HSH_" + currentEntityName + "_7."+currentExtension;
			currentAlias["Coeff8"] = "HSH_" + currentEntityName + "_8."+currentExtension;
			
			//full material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HSHNormMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//noalbedo material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HSHNormNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			
			//linear versions
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HSHNormLinMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HSHNormLinNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//lightmap versions
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HSHNormConstMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"HSHNormConstNoAlbedoMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentAlias.clear();

			// Zernike basis coefficient maps
			currentAlias["Coeff0"] = "Zern_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "Zern_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "Zern_" + currentEntityName + "_2."+currentExtension;
			currentAlias["Coeff3"] = "Zern_" + currentEntityName + "_3."+currentExtension;
			currentAlias["Coeff4"] = "Zern_" + currentEntityName + "_4."+currentExtension;
			currentAlias["Coeff5"] = "Zern_" + currentEntityName + "_5."+currentExtension;
			
			//full material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"ZernNormMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//noalbedo material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"ZernNormNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);
			
			//linear versions
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"ZernNormLinMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"ZernNormLinNoAlbedoMat"); 
			currentMaterial->applyTextureAliases(currentAlias);

			//lightmap versions
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"ZernNormConstMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"ZernNormConstNoAlbedoMat"); 
		    currentMaterial->applyTextureAliases(currentAlias);

			currentAlias.clear();

			// Spherical harmonics coefficient maps
			currentAlias["Coeff0"] = "SH_" + currentEntityName + "_0."+currentExtension;
			currentAlias["Coeff1"] = "SH_" + currentEntityName + "_1."+currentExtension;
			currentAlias["Coeff2"] = "SH_" + currentEntityName + "_2."+currentExtension;
			currentAlias["Coeff3"] = "SH_" + currentEntityName + "_3."+currentExtension;
			currentAlias["Coeff4"] = "SH_" + currentEntityName + "_4."+currentExtension;
			currentAlias["Coeff5"] = "SH_" + currentEntityName + "_5."+currentExtension;
			currentAlias["Coeff6"] = "SH_" + currentEntityName + "_6."+currentExtension;
			currentAlias["Coeff7"] = "SH_" + currentEntityName + "_7."+currentExtension;
			currentAlias["Coeff8"] = "SH_" + currentEntityName + "_8."+currentExtension;
			
			//full material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"SHNormMat");
			currentMaterial->applyTextureAliases(currentAlias);

			//noalbedo material
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"SHNormNoAlbedoMat");
			currentMaterial->applyTextureAliases(currentAlias);

			//linear versions
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"SHNormLinMat");
			currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"SHNormLinNoAlbedoMat");
			currentMaterial->applyTextureAliases(currentAlias);

			//lightmap versions
			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"SHNormConstMat");
		    currentMaterial->applyTextureAliases(currentAlias);

			currentMaterial = MaterialManager::getSingleton().getByName(currentEntityName+"SHNormConstNoAlbedoMat");
		    currentMaterial->applyTextureAliases(currentAlias);

			EntityIterator.moveNext();
		}




		return true;
	}


protected:

	SceneManager* mSceneMgr;

	Overlay* mHelpOverlay;

	OverlayElement* lightingMethod;
	OverlayElement* albedoState;
	OverlayElement* textureFilter;
	OverlayElement* textureCompression;
	
	HDRListener *hdrListener;
	
	bool mHelpOn;
	bool mAlbedoOn;
	TextureType mTextureType;
	bool mHDROn;
	

	BasisType mBasisType;


	bool changeMaterial(const BasisType rend,const bool albedo)
		{

		SceneManager::MovableObjectIterator EntityIterator = mSceneMgr->getMovableObjectIterator("Entity");
		Entity * currentEntity = NULL;

		while( EntityIterator.hasMoreElements() )
		{
			
			currentEntity = static_cast<Ogre::Entity *>(EntityIterator.peekNextValue());
			String currentEntityName=currentEntity->getName();

			String materialName = currentEntityName;

			if(albedo)
			{
				if(rend == MODHBASISCONSTANT)
				{	
					materialName += "HmodNormConstMat";
				}
				else if(rend == MODHBASISLINEAR)
				{	
					materialName += "HmodNormLinMat";
				}
				else if(rend == MODHBASISQUADRATIC)
				{	
					materialName += "HmodNormMat";
				}
				if(rend == HBASISCONSTANT)
				{	
					materialName += "HNormConstMat";
				}
				else if(rend == HBASISLINEAR)
				{	
					materialName += "HNormLinMat";
				}
				else if(rend == HBASISQUADRATIC)
				{	
					materialName += "HNormMat";
				}
				else if(rend == SHCONSTANT)
				{	
					materialName += "SHNormConstMat";
				}
				else if(rend == SHLINEAR)
				{	
					materialName += "SHNormLinMat";
				}
				else if(rend == SHQUADRATIC)
				{	
					materialName += "SHNormMat";
				}
				else if(rend == VOLUMESH)
				{	
					materialName += "SHNormVolumeMat";
				}
				else if(rend == HL2)
				{	
					materialName += "HL2NormMat";
				}
				else if(rend == HSHCONSTANT)
				{	
					materialName += "HSHNormConstMat";
				}
				else if(rend == HSHLINEAR)
				{	
					materialName += "HSHNormLinMat";
				}
				else if(rend == HSHQUADRATIC)
				{	
					materialName += "HSHNormMat";
				}
				else if(rend == ZERNCONSTANT)
				{	
					materialName += "ZernNormConstMat";
				}
				else if(rend == ZERNLINEAR)
				{	
					materialName += "ZernNormLinMat";
				}
				else if(rend == ZERNQUADRATIC)
				{	
					materialName += "ZernNormMat";
				}
				else if(rend == MAKCONSTANT)
				{	
					materialName += "MakNormConstMat";
				}
				else if(rend == MAKLINEAR)
				{	
					materialName += "MakNormLinMat";
				}
				else if(rend == MAKQUADRATIC)
				{	
					materialName += "MakNormMat";
				}
				else if(rend == MODHBASISIPOM)
				{	
					materialName += "IPOMMat";
				}
			}
			else
			{
				if(rend == MODHBASISCONSTANT)
				{	
					materialName += "HmodNormConstNoAlbedoMat";
				}
				else if(rend == MODHBASISLINEAR)
				{	
					materialName += "HmodNormLinNoAlbedoMat";
				}
				else if(rend == MODHBASISQUADRATIC)
				{	
					materialName += "HmodNormNoAlbedoMat";
				}
				if(rend == HBASISCONSTANT)
				{	
					materialName += "HNormConstNoAlbedoMat";
				}
				else if(rend == HBASISLINEAR)
				{	
					materialName += "HNormLinNoAlbedoMat";
				}
				else if(rend == HBASISQUADRATIC)
				{	
					materialName += "HNormNoAlbedoMat";
				}
				else if(rend == SHCONSTANT)
				{	
					materialName += "SHNormConstNoAlbedoMat";
				}
				else if(rend == SHLINEAR)
				{	
					materialName += "SHNormLinNoAlbedoMat";
				}
				else if(rend == SHQUADRATIC)
				{	
					materialName += "SHNormNoAlbedoMat";
				}
				else if(rend == VOLUMESH)
				{	
					materialName += "SHNormVolumeNoAlbedoMat";
				}
				else if(rend == HL2)
				{	
					materialName += "HL2NormNoAlbedoMat";
				}
				else if(rend == HSHCONSTANT)
				{	
					materialName += "HSHNormConstNoAlbedoMat";
				}
				else if(rend == HSHLINEAR)
				{	
					materialName += "HSHNormLinNoAlbedoMat";
				}
				else if(rend == HSHQUADRATIC)
				{	
					materialName += "HSHNormNoAlbedoMat";
				}
				else if(rend == ZERNCONSTANT)
				{	
					materialName += "ZernNormConstNoAlbedoMat";
				}
				else if(rend == ZERNLINEAR)
				{	
					materialName += "ZernNormLinNoAlbedoMat";
				}
				else if(rend == ZERNQUADRATIC)
				{	
					materialName += "ZernNormNoAlbedoMat";
				}
				else if(rend == MAKCONSTANT)
				{	
					materialName += "MakNormConstNoAlbedoMat";
				}
				else if(rend == MAKLINEAR)
				{	
					materialName += "MakNormLinNoAlbedoMat";
				}
				else if(rend == MAKQUADRATIC)
				{	
					materialName += "MakNormNoAlbedoMat";
				}
				else if(rend == MODHBASISIPOM)
				{	
					materialName += "IPOMNoAlbedoMat";
				}
			}

			MaterialPtr toAssignMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName(materialName);

			currentEntity->setMaterial(toAssignMaterial);
		

			EntityIterator.moveNext();
		}

	return true;

	}

	void registerCompositors(void)
	{

		hdrListener = new HDRListener();

		Ogre::Viewport *vp = mWindow->getViewport(0);

		Ogre::CompositorInstance *compInstance = Ogre::CompositorManager::getSingleton().addCompositor(vp, "HDR", -1);
		Ogre::CompositorManager::getSingleton().setCompositorEnabled(vp, "HDR", false);

		compInstance->addListener(hdrListener);
		hdrListener->notifyViewportSize(vp->getActualWidth(), vp->getActualHeight());
		hdrListener->notifyCompositor(compInstance);


	}

};
