function setup()

	bset("gather.sampletype", "indirect illumination")
	bset("gather.space", "tangent space")
	bset("gather.distribution", "equiareal")
	bset("gather.minsamples", 100)
	bset("gather.maxsamples", 100)
	bset("gather.coneangle", 180)
	bset("output.size", 12)

end


function bake()

	-- RNM basis vectors
	basis1 = vec3(0.8165, 0.0, 0.577)
	basis2 = vec3(-0.408, 0.707, 0.577)
	basis3 = vec3(-0.408, -0.707, 0.577)

	-- Project indirect light onto our RNM basis vectors
	rgb1 = getSampleProjectedSum(basis1, false)
	rgb2 = getSampleProjectedSum(basis2, false)
	rgb3 = getSampleProjectedSum(basis3, false)

	-- Weigh the indirect contribution
	n = getSampleCount()
	rgb1 = (rgb1 * 2.0) / n
	rgb2 = (rgb2 * 2.0) / n
	rgb3 = (rgb3 * 2.0) / n


	-- Add direct light contribution

	nlights = getLights()
	for i = 1, nlights do

		col = getLightCol(i)

		if (getLightAmb(i)) then
			rgb1 = rgb1 + col
			rgb2 = rgb2 + col
			rgb3 = rgb3 + col
		else
			localLightDir = normalize(worldToGather(getLightDir(i)))

			weight = dot3(basis1, localLightDir)
			if (weight > 0) then
				rgb1 = rgb1 + col * weight
			end
			weight = dot3(basis2, localLightDir)
			if (weight > 0) then
				rgb2 = rgb2 + col * weight
			end
			weight = dot3(basis3, localLightDir)
			if (weight > 0) then
				rgb3 = rgb3 + col * weight
			end
		end
	end

	array = {rgb1[1], rgb1[2], rgb1[3], 1.0,
		 rgb2[1], rgb2[2], rgb2[3], 1.0,
		 rgb3[1], rgb3[2], rgb3[3], 1.0}

	return array

end
