-- Globals
shbands = 3
shbasis = shbands*shbands

--This defines the range the linear and quadratic coefficients lie in: -compressRange..compressRange
--This can be set by hand or defined by the absolute maximum value in the textures [texturename]_1..5
--The textures need to be range decompressed by the application using this value
compressRange = 1

function setup()

	bset("gather.sampletype", "indirect illumination")
	bset("gather.space", "tangent space") --we need radiance in tangent space
	bset("gather.distribution", "equiareal")
	bset("gather.minsamples", 100)
	bset("gather.maxsamples", 500)
	bset("gather.useRadianceCache", true)
	bset("gather.coneangle", 360) --results look weird with 180
	
	--bset("gather.clamp", false) --set to false for HDR output

	bset("basis.type", "sh") --Turtle has SH build in
	bset("basis.rgb", true)
	bset("basis.sh.bands", shbands)

	bset("output.size", 36)

end

function bake() 


	ctR = getBasisCoefficients(1) --this function performs a Monte-Carlo-Integration, returning the SH coefficient vector
	ctG = getBasisCoefficients(2)
	ctB = getBasisCoefficients(3)

	nlights = getLights()  --the standard point lights are expanded into SH and added 
	for i = 1, nlights do
		col = getLightCol(i)
		if (not getLightAmb(i)) then
			localLightDir = normalize(worldToGather(getLightDir(i)))
			c = evalSHBasis(localLightDir, shbands)
			for j = 1, shbasis do
				ctR[j] = ctR[j] + c[j] * col[1]
				ctG[j] = ctG[j] + c[j] * col[2]
				ctB[j] = ctB[j] + c[j] * col[3]
			end
		end
	end
	
-- diffuse convolution using Funk-Hecke theorem
  
  diff_kernel = {1.0,2.0/3.0,1.0/4.0} --built in div by pi, so no pi in evaluation
  
  for l = 0, shbands-1 do --loop with index start 0
    for m = -l, l do
      
      ser = (l*(l+1)+m)+1 --correct for index start 1, LUA starts with 1
      ctR[ser] = ctR[ser] * diff_kernel[l+1] --correction for index start 1
      ctG[ser] = ctG[ser] * diff_kernel[l+1]
	  ctB[ser] = ctB[ser] * diff_kernel[l+1]
	  end
	end
	
--projection into HSH
	
	hshR = {}
	hshG = {}
	hshB = {}


	hshR[1] = 0.7071 * ctR[1] + 0.6123 * ctR[3]
	hshR[2] = -0.6738* ctR[2] - 0.6745 * ctR[6]
	hshR[3] = 0.3535 * ctR[3] + 0.6846 * ctR[7]
	hshR[4] = -0.6738* ctR[4] - 0.6745 * ctR[8]
	hshR[5] = 0.6187 * ctR[5] 
	hshR[6] = 0.1577 * ctR[2] - 0.2116 * ctR[6]
	hshR[7] = 0.1767 * ctR[7]
	hshR[8] = 0.1577 * ctR[4] - 0.2116 * ctR[8] 
	hshR[9] = 0.6187 * ctR[9]
	
	hshG[1] = 0.7071 * ctG[1] + 0.6123 * ctG[3]
	hshG[2] = -0.6738* ctG[2] - 0.6745 * ctG[6]
	hshG[3] = 0.3535 * ctG[3] + 0.6846 * ctG[7]
	hshG[4] = -0.6738* ctG[4] - 0.6745 * ctG[8]
	hshG[5] = 0.6187 * ctG[5] 
	hshG[6] = 0.1577 * ctG[2] - 0.2116 * ctG[6]
	hshG[7] = 0.1767 * ctG[7]
	hshG[8] = 0.1577 * ctG[4] - 0.2116 * ctG[8] 
	hshG[9] = 0.6187 * ctG[9]
	
	hshB[1] = 0.7071 * ctB[1] + 0.6123 * ctB[3]
	hshB[2] = -0.6738* ctB[2] - 0.6745 * ctB[6]
	hshB[3] = 0.3535 * ctB[3] + 0.6846 * ctB[7]
	hshB[4] = -0.6738* ctB[4] - 0.6745 * ctB[8]
	hshB[5] = 0.6187 * ctB[5] 
	hshB[6] = 0.1577 * ctB[2] - 0.2116 * ctB[6]
	hshB[7] = 0.1767 * ctB[7]
	hshB[8] = 0.1577 * ctB[4] - 0.2116 * ctB[8] 
	hshB[9] = 0.6187 * ctB[9]

		
--Convert first coefficient to sRGB
    hshR[1] = math.pow(hshR[1],1.0/2.2)
    hshG[1] = math.pow(hshG[1],1.0/2.2)
    hshB[1] = math.pow(hshB[1],1.0/2.2)
	

--range compressing with a global factor all but the constant term 
--These coefficients are transported in linear space
  for i = 2, 9 do
    hshR[i] = (hshR[i]+compressRange)/(2.0*compressRange)
    hshG[i] = (hshG[i]+compressRange)/(2.0*compressRange)
    hshB[i] = (hshB[i]+compressRange)/(2.0*compressRange)
  end

--neagtive clamp light map, sometimes there are values very slightly in the negative
  if hshR[1] <= 0.0 then hshR[1] = 0.0 end  
  if hshG[1] <= 0.0 then hshG[1] = 0.0 end 
  if hshB[1] <= 0.0 then hshB[1] = 0.0 end 
  
--output
    out = {}
    
    out[1] = hshR[1] -- contains light map
    out[2] = hshG[1]
    out[3] = hshB[1]
    out[4] = 1.0
    
    out[5] = hshR[2]
    out[6] = hshG[2]
    out[7] = hshB[2]
    out[8] = 1.0
    
    out[9] = hshR[3]
    out[10] = hshG[3]
    out[11] = hshB[3]
    out[12] = 1.0
    
    out[13] = hshR[4]
    out[14] = hshG[4]
    out[15] = hshB[4]
    out[16] = 1.0
    
    out[17] = hshR[5]
    out[18] = hshG[5]
    out[19] = hshB[5]
    out[20] = 1.0
	
    out[21] = hshR[6]
    out[22] = hshG[6]
    out[23] = hshB[6]
    out[24] = 1.0

    out[25] = hshR[7]
    out[26] = hshG[7]
    out[27] = hshB[7]
    out[28] = 1.0	

    out[29] = hshR[8]
    out[30] = hshG[8]
    out[31] = hshB[8]
    out[32] = 1.0

    out[33] = hshR[9]
    out[34] = hshG[9]
    out[35] = hshB[9]
    out[36] = 1.0
	
	return out
end
