-- Globals
shbands = 3
shbasis = shbands*shbands

--This defines the range the linear and quadratic coefficients lie in: -compressRange..compressRange
--This can be set by hand or defined by the absolute maximum value in the textures [texturename]_1..5
--The textures need to be range decompressed by the application using this value
compressRange = 1

function setup()

	bset("gather.sampletype", "indirect illumination")
	bset("gather.space", "tangent space") --we need radiance in tangent space
	bset("gather.distribution", "equiareal")
	bset("gather.minsamples", 100)
	bset("gather.maxsamples", 500)
	bset("gather.useRadianceCache", true)
	bset("gather.coneangle", 360) --results look weird with 180
	
	--bset("gather.clamp", false) --set to false for HDR output

	bset("basis.type", "sh") --Turtle has SH build in
	bset("basis.rgb", true)
	bset("basis.sh.bands", shbands)

	bset("output.size", 36)

end

function bake() 


	ctR = getBasisCoefficients(1) --this function performs a Monte-Carlo-Integration, returning the SH coefficient vector
	ctG = getBasisCoefficients(2)
	ctB = getBasisCoefficients(3)

	nlights = getLights()  --the standard point lights are expanded into SH and added 
	for i = 1, nlights do
		col = getLightCol(i)
		if (not getLightAmb(i)) then
			localLightDir = normalize(worldToGather(getLightDir(i)))
			c = evalSHBasis(localLightDir, shbands)
			for j = 1, shbasis do
				ctR[j] = ctR[j] + c[j] * col[1]
				ctG[j] = ctG[j] + c[j] * col[2]
				ctB[j] = ctB[j] + c[j] * col[3]
			end
		end
	end
	
-- diffuse convolution using Funk-Hecke theorem
  
  diff_kernel = {1.0,2.0/3.0,1.0/4.0} --built in div by pi, so no pi in evaluation
  
  for l = 0, shbands-1 do --loop with index start 0
    for m = -l, l do
      
      ser = (l*(l+1)+m)+1 --correct for index start 1, LUA starts with 1
      ctR[ser] = ctR[ser] * diff_kernel[l+1] --correction for index start 1
      ctG[ser] = ctG[ser] * diff_kernel[l+1]
	  ctB[ser] = ctB[ser] * diff_kernel[l+1]
	  end
	end
	
--projection into makhotkin
	
	makR = {}
	makG = {}
	makB = {}


	makR[1] = 0.6666 * ctR[1] + 0.6928 * ctR[3] + 0.2129 * ctR[7]
	makR[2] = 0.6106 * ctR[4] + 0.7056 * ctR[8]
	makR[3] = -0.1885* ctR[1] + 0.1399 * ctR[3] + 0.6625 * ctR[7]
	makR[4] = 0.6106 * ctR[2] + 0.7056 * ctR[6]
	makR[5] = 0.5541 * ctR[9] 
	makR[6] = -0.2539* ctR[4] + 0.0448 * ctR[8]
	makR[7] = 0.0989 * ctR[1] - 0.0119 * ctR[3] - 0.0603 * ctR[7]
	makR[8] =-0.2539 * ctR[2] + 0.0448 * ctR[6] 
	makR[9] = 0.5541 * ctR[5]
	
	makG[1] = 0.6666 * ctG[1] + 0.6928 * ctG[3] + 0.2129 * ctG[7]
	makG[2] = 0.6106 * ctG[4] + 0.7056 * ctG[8]
	makG[3] = -0.1885* ctG[1] + 0.1399 * ctG[3] + 0.6625 * ctG[7]
	makG[4] = 0.6106 * ctG[2] + 0.7056 * ctG[6]
	makG[5] = 0.5541 * ctG[9] 
	makG[6] = -0.2539* ctG[4] + 0.0448 * ctG[8]
	makG[7] = 0.0989 * ctG[1] - 0.0119 * ctG[3] - 0.0603 * ctG[7]
	makG[8] =-0.2539 * ctG[2] + 0.0448 * ctG[6] 
	makG[9] = 0.5541 * ctG[5]
	
	makB[1] = 0.6666 * ctB[1] + 0.6928 * ctB[3] + 0.2129 * ctB[7]
	makB[2] = 0.6106 * ctB[4] + 0.7056 * ctB[8]
	makB[3] = -0.1885* ctB[1] + 0.1399 * ctB[3] + 0.6625 * ctB[7]
	makB[4] = 0.6106 * ctB[2] + 0.7056 * ctB[6]
	makB[5] = 0.5541 * ctB[9] 
	makB[6] = -0.2539* ctB[4] + 0.0448 * ctB[8]
	makB[7] = 0.0989 * ctB[1] - 0.0119 * ctB[3] - 0.0603 * ctB[7]
	makB[8] =-0.2539 * ctB[2] + 0.0448 * ctB[6] 
	makB[9] = 0.5541 * ctB[5]

		
--Convert first coefficient to sRGB
    makR[1] = math.pow(makR[1],1.0/2.2)
    makG[1] = math.pow(makG[1],1.0/2.2)
    makB[1] = math.pow(makB[1],1.0/2.2)
	

--range compressing with a global factor all but the constant term 
--These coefficients are transported in linear space
  for i = 2, 9 do
    makR[i] = (makR[i]+compressRange)/(2.0*compressRange)
    makG[i] = (makG[i]+compressRange)/(2.0*compressRange)
    makB[i] = (makB[i]+compressRange)/(2.0*compressRange)
  end

--neagtive clamp light map, sometimes there are values very slightly in the negative
  if makR[1] <= 0.0 then makR[1] = 0.0 end  
  if makG[1] <= 0.0 then makG[1] = 0.0 end 
  if makB[1] <= 0.0 then makB[1] = 0.0 end 
  
--output
    out = {}
    
    out[1] = makR[1] -- contains light map
    out[2] = makG[1]
    out[3] = makB[1]
    out[4] = 1.0
    
    out[5] = makR[2]
    out[6] = makG[2]
    out[7] = makB[2]
    out[8] = 1.0
    
    out[9] = makR[3]
    out[10] = makG[3]
    out[11] = makB[3]
    out[12] = 1.0
    
    out[13] = makR[4]
    out[14] = makG[4]
    out[15] = makB[4]
    out[16] = 1.0
    
    out[17] = makR[5]
    out[18] = makG[5]
    out[19] = makB[5]
    out[20] = 1.0
	
    out[21] = makR[6]
    out[22] = makG[6]
    out[23] = makB[6]
    out[24] = 1.0

    out[25] = makR[7]
    out[26] = makG[7]
    out[27] = makB[7]
    out[28] = 1.0	

    out[29] = makR[8]
    out[30] = makG[8]
    out[31] = makB[8]
    out[32] = 1.0

    out[33] = makR[9]
    out[34] = makG[9]
    out[35] = makB[9]
    out[36] = 1.0
	
	return out
end
