-- Globals
shbands = 3
shbasis = shbands*shbands

--This defines the range the linear and quadratic coefficients lie in: -compressRange..compressRange
--This can be set by hand or defined by the absolute maximum value in the textures [texturename]_1..5
--The textures need to be range decompressed by the application using this value
compressRange = 1

function setup()

	bset("gather.sampletype", "indirect illumination")
	bset("gather.space", "tangent space") --we need radiance in tangent space
	bset("gather.distribution", "equiareal")
	bset("gather.minsamples", 100)
	bset("gather.maxsamples", 500)
	bset("gather.useRadianceCache", true)
	bset("gather.coneangle", 360) --results look weird with 180
	
	--bset("gather.clamp", false) --set to false for HDR output

	bset("basis.type", "sh") --Turtle has SH build in
	bset("basis.rgb", true)
	bset("basis.sh.bands", shbands)

	bset("output.size", 36)

end

function bake() 


	ctR = getBasisCoefficients(1) --this function performs a Monte-Carlo-Integration, returning the SH coefficient vector
	ctG = getBasisCoefficients(2)
	ctB = getBasisCoefficients(3)

	nlights = getLights()  --the standard point lights are expanded into SH and added 
	for i = 1, nlights do
		col = getLightCol(i)
		if (not getLightAmb(i)) then
			localLightDir = normalize(worldToGather(getLightDir(i)))
			c = evalSHBasis(localLightDir, shbands)
			for j = 1, shbasis do
				ctR[j] = ctR[j] + c[j] * col[1]
				ctG[j] = ctG[j] + c[j] * col[2]
				ctB[j] = ctB[j] + c[j] * col[3]
			end
		end
	end
	
-- diffuse convolution using Funk-Hecke theorem
  
  diff_kernel = {1.0,2.0/3.0,1.0/4.0} --built in div by pi, so no pi in evaluation
  
  for l = 0, shbands-1 do --loop with index start 0
    for m = -l, l do
      
      ser = (l*(l+1)+m)+1 --correct for index start 1, LUA starts with 1
      ctR[ser] = ctR[ser] * diff_kernel[l+1] --correction for index start 1
      ctG[ser] = ctG[ser] * diff_kernel[l+1]
	  ctB[ser] = ctB[ser] * diff_kernel[l+1]
	  end
	end
--Convert first coefficient to sRGB
    ctR[1] = math.pow(ctR[1],1.0/2.2)
    ctG[1] = math.pow(ctG[1],1.0/2.2)
    ctB[1] = math.pow(ctB[1],1.0/2.2)

--range compressing all but the constant term
  for i = 2, 9 do
    ctR[i] = (ctR[i]+compressRange)/(2.0*compressRange)
    ctG[i] = (ctG[i]+compressRange)/(2.0*compressRange)
    ctB[i] = (ctB[i]+compressRange)/(2.0*compressRange)
  end

--neagtive clamp light map, sometimes there are values very slightly in the negative
  if ctR[1] <= 0.0 then ctR[1] = 0.0 end  
  if ctG[1] <= 0.0 then ctG[1] = 0.0 end 
  if ctB[1] <= 0.0 then ctB[1] = 0.0 end 

--output
    out = {}
    
    out[1] = ctR[1] -- contains constant term
	out[2] = ctG[1]
	out[3] = ctB[1]
    out[4] = 1.0
    
    out[5] = ctR[2]
	out[6] = ctG[2]
	out[7] = ctB[2]
    out[8] = 1.0
    
    out[9] = ctR[3]
	out[10] = ctG[3]
	out[11] = ctB[3]
    out[12] = 1.0
    
    out[13] = ctR[4]
	out[14] = ctG[4]
	out[15] = ctB[4]
    out[16] = 1.0
    
    out[17] = ctR[5]
	out[18] = ctG[5]
	out[19] = ctB[5]
    out[20] = 1.0
	
    out[21] = ctR[6]
	out[22] = ctG[6]
	out[23] = ctB[6]
    out[24] = 1.0

    out[25] = ctR[7]
	out[26] = ctG[7]
	out[27] = ctB[7]
    out[28] = 1.0
    
    out[29] = ctR[8]
	out[30] = ctG[8]
	out[31] = ctB[8]
    out[32] = 1.0
	
    out[33] = ctR[9]
	out[34] = ctG[9]
	out[35] = ctB[9]
    out[36] = 1.0
	
	return out
end
