-- Globals
shbands = 3
shbasis = shbands*shbands

--This defines the range the linear and quadratic coefficients lie in: -compressRange..compressRange
--This can be set by hand or defined by the absolute maximum value in the textures [texturename]_1..5
--The textures need to be range decompressed by the application using this value
compressRange = 1

function setup()

	bset("gather.sampletype", "indirect illumination")
	bset("gather.space", "tangent space") --we need radiance in tangent space
	bset("gather.distribution", "equiareal")
	bset("gather.minsamples", 100)
	bset("gather.maxsamples", 500)
	bset("gather.useRadianceCache", true)
	bset("gather.coneangle", 360) --results look weird with 180
	
	--bset("gather.clamp", false) --set to false for HDR output

	bset("basis.type", "sh") --Turtle has SH build in
	bset("basis.rgb", true)
	bset("basis.sh.bands", shbands)

	bset("output.size", 24)

end

function bake() 


	ctR = getBasisCoefficients(1) --this function performs a Monte-Carlo-Integration, returning the SH coefficient vector
	ctG = getBasisCoefficients(2)
	ctB = getBasisCoefficients(3)

	nlights = getLights()  --the standard point lights are expanded into SH and added 
	for i = 1, nlights do
		col = getLightCol(i)
		if (not getLightAmb(i)) then
			localLightDir = normalize(worldToGather(getLightDir(i)))
			c = evalSHBasis(localLightDir, shbands)
			for j = 1, shbasis do
				ctR[j] = ctR[j] + c[j] * col[1]
				ctG[j] = ctG[j] + c[j] * col[2]
				ctB[j] = ctB[j] + c[j] * col[3]
			end
		end
	end
	
-- diffuse convolution using Funk-Hecke theorem
  
  diff_kernel = {1.0,2.0/3.0,1.0/4.0} --built in div by pi, so no pi in evaluation
  
  for l = 0, shbands-1 do --loop with index start 0
    for m = -l, l do
      
      ser = (l*(l+1)+m)+1 --correct for index start 1, LUA starts with 1
      ctR[ser] = ctR[ser] * diff_kernel[l+1] --correction for index start 1
      ctG[ser] = ctG[ser] * diff_kernel[l+1]
	  ctB[ser] = ctB[ser] * diff_kernel[l+1]
	  end
	end
	
--projection into the Zernike basis
	
	ZernR = {}
	ZernG = {}
	ZernB = {}

	ZernR[1] = 0.7071 * ctR[1] + 0.6123 * ctR[3]
	ZernR[2] = -0.7044* ctR[2] - 0.5574 * ctR[6]
	ZernR[3] = 0.7044 * ctR[4] + 0.5574 * ctR[8]
	ZernR[4] = -0.6987* ctR[5]
	ZernR[5] = -0.3535* ctR[3] - 0.6846 * ctR[7]
	ZernR[6] = 0.6987 * ctR[9]
	
	ZernG[1] = 0.7071 * ctG[1] + 0.6123 * ctG[3]
	ZernG[2] = -0.7044* ctG[2] - 0.5574 * ctG[6]
	ZernG[3] = 0.7044 * ctG[4] + 0.5574 * ctG[8]
	ZernG[4] = -0.6987* ctG[5]
	ZernG[5] = -0.3535* ctG[3] - 0.6846 * ctG[7]
	ZernG[6] = 0.6987 * ctG[9]

	ZernB[1] = 0.7071 * ctB[1] + 0.6123 * ctB[3]
	ZernB[2] = -0.7044* ctB[2] - 0.5574 * ctB[6]
	ZernB[3] = 0.7044 * ctB[4] + 0.5574 * ctB[8]
	ZernB[4] = -0.6987* ctB[5]
	ZernB[5] = -0.3535* ctB[3] - 0.6846 * ctB[7]
	ZernB[6] = 0.6987 * ctB[9]
	
--Convert first coefficient to sRGB
    ZernR[1] = math.pow(ZernR[1],1.0/2.2)
    ZernG[1] = math.pow(ZernG[1],1.0/2.2)
    ZernB[1] = math.pow(ZernB[1],1.0/2.2)

--range compressing with a global factor all but the constant term 
--These coefficients are transported in linear space
  for i = 2, 6 do
    ZernR[i] = (ZernR[i]+compressRange)/(2.0*compressRange)
    ZernG[i] = (ZernG[i]+compressRange)/(2.0*compressRange)
    ZernB[i] = (ZernB[i]+compressRange)/(2.0*compressRange)
  end

--neagtive clamp light map, sometimes there are values very slightly in the negative
  if ZernR[1] <= 0.0 then ZernR[1] = 0.0 end  
  if ZernG[1] <= 0.0 then ZernG[1] = 0.0 end 
  if ZernB[1] <= 0.0 then ZernB[1] = 0.0 end 
  
--output
    out = {}
    
    out[1] = ZernR[1] -- contains light map
	out[2] = ZernG[1]
	out[3] = ZernB[1]
    out[4] = 1.0
    
    out[5] = ZernR[2]
	out[6] = ZernG[2]
	out[7] = ZernB[2]
  out[8] = 1.0
    
    out[9] = ZernR[3]
	out[10] = ZernG[3]
	out[11] = ZernB[3]
   out[12] = 1.0
    
    out[13] = ZernR[4]
	out[14] = ZernG[4]
	out[15] = ZernB[4]
    out[16] = 1.0
    
    out[17] = ZernR[5]
	out[18] = ZernG[5]
	out[19] = ZernB[5]
    out[20] = 1.0
	
	out[21] = ZernR[6]
	out[22] = ZernG[6]
	out[23] = ZernB[6]
    out[24] = 1.0
	
	return out
end
