#include <iostream>
#include <cstdio>
#include <cassert>
#include <stdlib.h>
#include <string.h>

#include <windows.h>
#include <d3d9.h>
#include <d3dx9.h>
 
#include <IL\il.h>
#include <IL\ilu.h>

#define TIXML_USE_STL

#include <tinyxml\tinyxml.h>

typedef unsigned char UINT8;
typedef unsigned short UINT16;

LPDIRECT3D9 gD3dObject=NULL;
LPDIRECT3DDEVICE9 gD3dDevice=NULL;

int checkError(HRESULT hr)
{
    if(FAILED(hr))
    {
        std::cerr << "D3D9 failure" << std::endl;
        exit(0);
    }
}

LRESULT WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
    return DefWindowProc( hWnd, uMsg, wParam, lParam );
}

int GetValue(TiXmlElement* xml_element, int index) 
{
	TiXmlElement* xml_active_element = xml_element->FirstChildElement("values")->FirstChildElement("f");
	for(int i=1; i<index; i++) 
	{
		xml_active_element = xml_active_element->NextSiblingElement("f");
	}
	return 255*atof(xml_active_element->GetText())+0.5f;
}

UINT8* GetData(int index) 
{
	TiXmlDocument *xml_file = new TiXmlDocument("sh_polySurfaceShape1.xml");
	bool loadOkay = xml_file->LoadFile();

	TiXmlElement *xml_header = 0;
	xml_header = xml_file->FirstChildElement("pointCloud")->FirstChildElement("header");
	TiXmlElement *xml_num_points = 0;
	xml_num_points = xml_header->FirstChildElement("numPoints");
	int num_points = atoi(xml_num_points->GetText()); // cubic root here, textures have to be cubic e.g. 32x32x32
	TiXmlElement *xml_vertex_array = 0;
	xml_vertex_array = xml_header->NextSiblingElement("body")->FirstChildElement("floatArrayPoint");
	TiXmlElement *xml_current_element = 0;
	xml_current_element = xml_vertex_array;
	//--------------------------------
	int width = 16;
	int height = 16;
	int depth = 8;
	int memory_needed = width * height * depth * 4 * sizeof(unsigned char);
	ILubyte * data = (ILubyte *)malloc(memory_needed);
	UINT8* image_data = (UINT8*)data;
	//--------------------------------
	bool next_line = false;
	bool even_line = false;
	for(int i = 0; i < width * height * depth * 4-width*4; i+=8) {
		if(next_line) 
		{
			even_line = !even_line;
			if(even_line) {
				i+=width*4;
				even_line = !even_line;
			}
			next_line = false;
		}
			if(i!=0)
				xml_current_element = xml_current_element->NextSiblingElement("floatArrayPoint");
			image_data[i] = GetValue(xml_current_element, 4*index+3);
			image_data[i+1] = GetValue(xml_current_element, 4*index+2);
			image_data[i+2] = GetValue(xml_current_element, 4*index+1);
			image_data[i+3] = GetValue(xml_current_element, 4*index+4);
			xml_current_element = xml_current_element->NextSiblingElement("floatArrayPoint");
			image_data[i+width*4] = GetValue(xml_current_element, 4*index+3);
			image_data[i+1+width*4] = GetValue(xml_current_element, 4*index+2);
			image_data[i+2+width*4] = GetValue(xml_current_element, 4*index+1);
			image_data[i+3+width*4] = GetValue(xml_current_element, 4*index+4);
			xml_current_element = xml_current_element->NextSiblingElement("floatArrayPoint");
			image_data[i+4+width*4] = GetValue(xml_current_element, 4*index+3);
			image_data[i+5+width*4] = GetValue(xml_current_element, 4*index+2);
			image_data[i+6+width*4] = GetValue(xml_current_element, 4*index+1);
			image_data[i+7+width*4] = GetValue(xml_current_element, 4*index+4);
			xml_current_element = xml_current_element->NextSiblingElement("floatArrayPoint");
			image_data[i+4] = GetValue(xml_current_element, 4*index+3);
			image_data[i+5] = GetValue(xml_current_element, 4*index+2);
			image_data[i+6] = GetValue(xml_current_element, 4*index+1);
			image_data[i+7] = GetValue(xml_current_element, 4*index+4);
		if((i+8)%(width*4)==0) 
		{
			next_line = true;
		}
	}
	return image_data;
}

int WinMain(
    HINSTANCE hInst,
    HINSTANCE hPrevInstance,
    LPSTR lpCmdLine,
    int nCmdShow
)
{

    size_t width, height, depth;

    std::cerr << "Main begins" << std::endl;
    ilInit();
    ilEnable( IL_FILE_OVERWRITE );

    std::cerr << "Creating window" << std::endl;
    std::string name = "Window";

    // Register the window class
    // NB allow 4 bytes of window data for D3D9RenderWindow pointer
    WNDCLASS wndClass = { CS_HREDRAW | CS_VREDRAW, (WNDPROC)WndProc, 0, 0, hInst,
        LoadIcon( NULL, IDI_APPLICATION ),
        LoadCursor( NULL, IDC_ARROW ),
        (HBRUSH)GetStockObject( BLACK_BRUSH ), NULL,
        TEXT(name.c_str()) };
    RegisterClass( &wndClass );

    // Create our main window
    // Pass pointer to self
    HWND hWnd = CreateWindow(TEXT(name.c_str()),
                                TEXT(name.c_str()),
                                WS_OVERLAPPEDWINDOW, 0, 0,
                                640, 480, 0L, 0L, hInst, 0);

    ShowWindow(hWnd, SW_SHOWNORMAL);
    UpdateWindow(hWnd);

    std::cerr << "Creating D3D9" << std::endl;
    gD3dObject = Direct3DCreate9(D3D_SDK_VERSION);

    D3DPRESENT_PARAMETERS presParams;
    ZeroMemory(&presParams, sizeof(presParams));
    presParams.Windowed = TRUE;
    presParams.BackBufferFormat = D3DFMT_UNKNOWN;
    presParams.BackBufferCount = 1;
    presParams.SwapEffect = D3DSWAPEFFECT_DISCARD;
    presParams.hDeviceWindow = hWnd;

    checkError(gD3dObject->CreateDevice(D3DADAPTER_DEFAULT,
        D3DDEVTYPE_HAL,hWnd,D3DCREATE_HARDWARE_VERTEXPROCESSING,
        &presParams, &gD3dDevice));

    ILuint ImageName;

    ilGenImages( 1, &ImageName );
    ilBindImage( ImageName );

    // Put it right side up
    ilEnable(IL_ORIGIN_SET);
    ilSetInteger(IL_ORIGIN_MODE, IL_ORIGIN_UPPER_LEFT);

	width = 16;
	height = 16;
	depth = 8;
    std::cerr << "Dimensions " << width << " " << height << " " << depth << std::endl;

	char buffer[2];
	for(int i=0; i<9; i++) 
	{
		char *outfile=new char[12];
		*outfile=0;
		strcat(outfile,"volume_sh_");
		strcat(outfile,itoa(i,buffer,10));
		strcat(outfile,".dds");
	std::cerr << "Creating volume texture" << std::endl;
    LPDIRECT3DVOLUMETEXTURE9 pVolumeTexture;

    checkError(D3DXCreateVolumeTexture(
        gD3dDevice,
        width, height, depth, 1, 0,
        D3DFMT_A8R8G8B8,
        D3DPOOL_MANAGED,
        &pVolumeTexture
    ));

    std::cerr << "Converting image" << std::endl;
    // Data
    unsigned char *data = (unsigned char*)GetData(i);

    std::cerr << "Acquiring volume" << std::endl;
    D3DBOX srcBox, destBox;
    LPDIRECT3DVOLUME9 volume;
    checkError(pVolumeTexture->GetVolumeLevel(0, &volume));

    std::cerr << "Loading image into D3DX" << std::endl;
    srcBox.Left = 0;
    srcBox.Top = 0;
    srcBox.Right = width;
    srcBox.Bottom = height;
    srcBox.Front = 0;
    srcBox.Back = depth;
    destBox = srcBox;
    unsigned int *datax = (unsigned int*)data;

    int rowPitch = width*4;
    int slicePitch = rowPitch*height;

    checkError(
        D3DXLoadVolumeFromMemory(
            volume,
            0,
            0,
            datax,
            D3DFMT_A8R8G8B8,
            rowPitch, slicePitch, 0,
            &srcBox,
            D3DX_DEFAULT, 0
        )
    );
    std::cerr << "Writing data" << std::endl;
    D3DXSaveVolumeToFile(outfile, D3DXIFF_DDS, volume, 0, 0);
    std::cerr << "Done" << std::endl;
	}
    ilDeleteImages(1, &ImageName);
}